@@ -83,8 +83,8 @@ static void     nautilus_tree_view_update_all_icons  (NautilusTreeView      *vie
 static void     cancel_possible_activation           (NautilusTreeView      *view);
 static void     nautilus_tree_view_update_model_node (NautilusTreeView      *view,
 						      NautilusTreeNode      *node);
-static void     nautilus_tree_view_initialize_class  (NautilusTreeViewClass *klass);
-static void     nautilus_tree_view_initialize        (NautilusTreeView      *view);
+static void     nautilus_tree_view_class_init        (NautilusTreeViewClass *klass);
+static void     nautilus_tree_view_init              (NautilusTreeView      *view);
 static void     register_unparented_node             (NautilusTreeView      *view,
 						      NautilusTreeNode      *node);
 static void     forget_unparented_node               (NautilusTreeView      *view,
@@ -94,7 +94,7 @@ static void     insert_unparented_nodes              (NautilusTreeView      *vie
 static void     expand_uri_sequence_and_select_end   (NautilusTreeView      *view);
 static gboolean is_anti_aliased			     (NautilusTreeView	    *view);
 
-EEL_DEFINE_CLASS_BOILERPLATE (NautilusTreeView,
+EEL_CLASS_BOILERPLATE (NautilusTreeView,
 				   nautilus_tree_view,
 				   NAUTILUS_TYPE_VIEW)
 
@@ -237,11 +237,9 @@ nautilus_tree_view_insert_model_node (NautilusTreeView *view, NautilusTreeNode *
 		if (nautilus_tree_view_model_node_to_view_node (view, node) == NULL) {
 
 			closed_pixbuf = nautilus_icon_factory_get_pixbuf_for_file
-				(file, NULL, NAUTILUS_ICON_SIZE_FOR_MENUS,
-				 is_anti_aliased (view));
+				(file, NULL, NAUTILUS_ICON_SIZE_FOR_MENUS);
 			open_pixbuf = nautilus_icon_factory_get_pixbuf_for_file
-				(file, "accept", NAUTILUS_ICON_SIZE_FOR_MENUS,
-				 is_anti_aliased (view));
+				(file, "accept", NAUTILUS_ICON_SIZE_FOR_MENUS);
 
 			view->details->inserting_node = TRUE;
 			EEL_CLIST_UNSET_FLAG (EEL_CLIST (view->details->tree),
@@ -258,8 +256,8 @@ nautilus_tree_view_insert_model_node (NautilusTreeView *view, NautilusTreeNode *
 					    CLIST_AUTO_SORT);
 			view->details->inserting_node = FALSE;
 
-			gdk_pixbuf_unref (closed_pixbuf);
-			gdk_pixbuf_unref (open_pixbuf);
+			g_object_unref (closed_pixbuf);
+			g_object_unref (open_pixbuf);
 
 
 			eel_ctree_node_set_row_data (EEL_CTREE (view->details->tree),
@@ -431,11 +429,9 @@ nautilus_tree_view_update_model_node (NautilusTreeView *view, NautilusTreeNode *
 		link_view_node_with_uri (view, view_node, nautilus_file_get_uri (file));
 
 		closed_pixbuf = nautilus_icon_factory_get_pixbuf_for_file
-			(file, NULL, NAUTILUS_ICON_SIZE_FOR_MENUS,
-			 is_anti_aliased (view));
+			(file, NULL, NAUTILUS_ICON_SIZE_FOR_MENUS);
 		open_pixbuf = nautilus_icon_factory_get_pixbuf_for_file
-			(file, "accept", NAUTILUS_ICON_SIZE_FOR_MENUS,
-			 is_anti_aliased (view));
+			(file, "accept", NAUTILUS_ICON_SIZE_FOR_MENUS);
 
 		name = nautilus_file_get_display_name (file);
 	
@@ -451,8 +447,8 @@ nautilus_tree_view_update_model_node (NautilusTreeView *view, NautilusTreeNode *
 
 		g_free (name);
 
-		gdk_pixbuf_unref (closed_pixbuf);
-		gdk_pixbuf_unref (open_pixbuf);
+		g_object_unref (closed_pixbuf);
+		g_object_unref (open_pixbuf);
 
 
 
@@ -490,7 +486,7 @@ register_unparented_node (NautilusTreeView *view, NautilusTreeNode *node)
 	g_return_if_fail (!nautilus_tree_node_is_toplevel (node));
 
 	if (g_list_find (view->details->unparented_tree_nodes, node) == NULL) {
-		gtk_object_ref (GTK_OBJECT (node));
+		g_object_ref (node);
 		view->details->unparented_tree_nodes = g_list_prepend (view->details->unparented_tree_nodes, node);
 	}
 }
@@ -503,7 +499,7 @@ forget_unparented_node (NautilusTreeView *view, NautilusTreeNode *node)
 
 	if (g_list_find (view->details->unparented_tree_nodes, node) != NULL) {
 		view->details->unparented_tree_nodes = g_list_remove (view->details->unparented_tree_nodes, node);
-		gtk_object_unref (GTK_OBJECT (node));
+		g_object_unref (node);
 	}
 }
 
@@ -535,7 +531,7 @@ insert_unparented_nodes (NautilusTreeView *view, NautilusTreeNode *node)
 				sub_node = p->data;
 				view->details->unparented_tree_nodes = g_list_remove (view->details->unparented_tree_nodes, sub_node);
 				nautilus_tree_view_insert_model_node (view, sub_node);
-				gtk_object_unref (GTK_OBJECT (sub_node));
+				g_object_unref (sub_node);
 			}
 			g_list_free (to_add);
 		}
@@ -783,21 +779,21 @@ nautilus_tree_view_load_from_filesystem (NautilusTreeView *view)
 					 nautilus_tree_view_model_node_changed_callback,
 					 view);
 
-	gtk_signal_connect (GTK_OBJECT (view->details->model),
+	g_signal_connect (view->details->model,
 			    "node_changed",
-			    nautilus_tree_view_model_node_changed_callback,
+			    G_CALLBACK (nautilus_tree_view_model_node_changed_callback),
 			    view);
-	gtk_signal_connect (GTK_OBJECT (view->details->model),
+	g_signal_connect (view->details->model,
 			    "node_removed",
-			    nautilus_tree_view_model_node_removed_callback,
+			    G_CALLBACK (nautilus_tree_view_model_node_removed_callback),
 			    view);
-	gtk_signal_connect (GTK_OBJECT (view->details->model),
+	g_signal_connect (view->details->model,
 			    "node_being_renamed",
-			    nautilus_tree_view_model_node_renamed_callback,
+			    G_CALLBACK (nautilus_tree_view_model_node_renamed_callback),
 			    view);
-	gtk_signal_connect (GTK_OBJECT (view->details->model),
+	g_signal_connect (view->details->model,
 			    "done_loading_children",
-			    nautilus_tree_view_model_done_loading_callback,
+			    G_CALLBACK (nautilus_tree_view_model_done_loading_callback),
 			    view);
 }
 
@@ -886,7 +882,7 @@ ctree_compare_rows (EelCList *clist,
 	if (compare_cached_key == clist) {
 		view = compare_cached_value;
 	} else {
-		view = gtk_object_get_data (GTK_OBJECT (clist), "tree_view");
+		view = g_object_get_data (G_OBJECT (clist), "tree_view");
 		compare_cached_key = clist;
 		compare_cached_value = view;
 	}
@@ -919,7 +915,7 @@ create_tree (NautilusTreeView *view)
 	/* set up ctree */
 	view->details->tree = eel_ctree_new (1, 0);
 
-	gtk_object_set_data (GTK_OBJECT (view->details->tree), "tree_view", (gpointer) view);
+	g_object_set_data (G_OBJECT (view->details->tree), "tree_view", (gpointer) view);
 	gtk_widget_add_events (GTK_WIDGET (view->details->tree), GDK_POINTER_MOTION_MASK);
 	
         eel_clist_set_selection_mode (EEL_CLIST (view->details->tree), GTK_SELECTION_SINGLE);
@@ -940,30 +936,33 @@ create_tree (NautilusTreeView *view)
 	eel_clist_set_column_auto_resize (EEL_CLIST (view->details->tree), 0, TRUE);
 	eel_clist_columns_autosize (EEL_CLIST (view->details->tree));
 	eel_clist_set_reorderable (EEL_CLIST (view->details->tree), FALSE);
+
+#ifdef GNOME2_CONVERSION_COMPLETE
 	eel_clist_set_row_height (EEL_CLIST (view->details->tree),
 				  MAX (NAUTILUS_ICON_SIZE_FOR_MENUS,
 				       view->details->tree->style->font->ascent
 				       + view->details->tree->style->font->descent));
+#endif
         eel_ctree_set_indent (EEL_CTREE (view->details->tree), 12);
 
-	gtk_signal_connect (GTK_OBJECT (view->details->tree),
+	g_signal_connect (view->details->tree,
 			    "tree_expand",
-			    GTK_SIGNAL_FUNC (tree_expand_callback), 
+			    G_CALLBACK (tree_expand_callback), 
 			    view);
 	
-	gtk_signal_connect (GTK_OBJECT (view->details->tree),
+	g_signal_connect (view->details->tree,
 			    "tree_collapse",
-			    GTK_SIGNAL_FUNC (tree_collapse_callback), 
+			    G_CALLBACK (tree_collapse_callback), 
 			    view);
 	
-	gtk_signal_connect (GTK_OBJECT (view->details->tree),
+	g_signal_connect (view->details->tree,
 			    "tree_select_row",
-			    GTK_SIGNAL_FUNC (tree_select_row_callback), 
+			    G_CALLBACK (tree_select_row_callback), 
 			    view);
 	
 	gtk_signal_connect_after (GTK_OBJECT (view->details->tree),
 				  "size_allocate",
-				  GTK_SIGNAL_FUNC (size_allocate_callback), 
+				  G_CALLBACK (size_allocate_callback), 
 				  view);
 	
 	/* Keep track of changes in these prefs to filter files accordingly. */
@@ -986,11 +985,11 @@ create_tree (NautilusTreeView *view)
 	
 	nautilus_tree_view_load_from_filesystem (view);
 
-	gtk_signal_connect_object_while_alive
-		(nautilus_icon_factory_get (),
-		 "icons_changed",
-		 nautilus_tree_view_update_all_icons,
-		 GTK_OBJECT (view));
+	g_signal_connect_closure_by_id (nautilus_icon_factory_get (),
+			                g_signal_lookup ("icons_changed", G_OBJECT_TYPE (nautilus_icon_factory_get())), 0,
+					g_cclosure_new_object_swap (G_CALLBACK (nautilus_tree_view_update_all_icons), 
+								    G_OBJECT (view)), 
+					FALSE);
 
 	nautilus_tree_view_init_dnd (view);
 
@@ -1001,38 +1000,33 @@ create_tree (NautilusTreeView *view)
 }
 
 static void
-tree_map_callback (GtkWidget *widget,
-		   gpointer callback_data)
+tree_activate_callback (BonoboControl *control,
+			gboolean       activating,
+			gpointer       user_data)
 {
 	NautilusTreeView *view;
 
-	view = NAUTILUS_TREE_VIEW (callback_data);
-	g_assert (widget == view->details->scrolled_window);
+	view = NAUTILUS_TREE_VIEW (user_data);
+	g_assert (view != NULL);
 
-	if (view->details->tree == NULL) {
-		create_tree (view);
+	if (activating) {
+		if (view->details->tree == NULL) {
+			create_tree (view);
+		}
+		nautilus_tree_model_set_defer_notifications (view->details->model, FALSE);
+		schedule_pending_idle_callback (view);
+		expand_uri_sequence_and_select_end (view);
+	} else {
+		nautilus_tree_model_set_defer_notifications (view->details->model, TRUE);
+		unschedule_pending_idle_callback (view);
 	}
-	nautilus_tree_model_set_defer_notifications (view->details->model, FALSE);
-	schedule_pending_idle_callback (view);
-	expand_uri_sequence_and_select_end (view);
 }
 
 static void
-tree_unmap_callback (GtkWidget *widget,
-		     gpointer callback_data)
+nautilus_tree_view_init (NautilusTreeView *view)
 {
-	NautilusTreeView *view;
+	BonoboControl *control;
 
-	view = NAUTILUS_TREE_VIEW (callback_data);
-	g_assert (widget == view->details->scrolled_window);
-
-	nautilus_tree_model_set_defer_notifications (view->details->model, TRUE);
-	unschedule_pending_idle_callback (view);
-}
-
-static void
-nautilus_tree_view_initialize (NautilusTreeView *view)
-{
 	view->details = g_new0 (NautilusTreeViewDetails, 1);
 
 	/* Obtain the filtering preferences */
@@ -1045,30 +1039,24 @@ nautilus_tree_view_initialize (NautilusTreeView *view)
 	view->details->show_non_directories = 
 		! eel_preferences_get_boolean (NAUTILUS_PREFERENCES_TREE_SHOW_ONLY_DIRECTORIES);
 
-	gtk_signal_connect (GTK_OBJECT (view),
-			    "load_location",
-			    GTK_SIGNAL_FUNC (tree_load_location_callback),
-			    view);
+	g_signal_connect (view,
+			  "load_location",
+			  G_CALLBACK (tree_load_location_callback),
+			  view);
 
 	view->details->scrolled_window = gtk_scrolled_window_new (NULL, NULL);
-	
+
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (view->details->scrolled_window), 
 					GTK_POLICY_AUTOMATIC,
 					GTK_POLICY_AUTOMATIC);
 						
 	gtk_widget_show (view->details->scrolled_window);
 
-	nautilus_view_construct (NAUTILUS_VIEW (view),
-				 view->details->scrolled_window);
+	control = bonobo_control_new (view->details->scrolled_window);
+	nautilus_view_construct_from_bonobo_control (NAUTILUS_VIEW (view), control);
 
-	gtk_signal_connect (GTK_OBJECT (view->details->scrolled_window),
-			    "map",
-			    tree_map_callback,
-			    view);
-	gtk_signal_connect (GTK_OBJECT (view->details->scrolled_window),
-			    "unmap",
-			    tree_unmap_callback,
-			    view);
+	g_signal_connect (control, "activate",
+			  G_CALLBACK (tree_activate_callback), view);
 }
 
 static void
@@ -1098,7 +1086,7 @@ free_view_node_to_uri_map_entry (gpointer key, gpointer value, gpointer callback
 }
 
 static void
-nautilus_tree_view_destroy (GtkObject *object)
+nautilus_tree_view_destroy (BonoboObject *object)
 {
 	NautilusTreeView *view;
 	
@@ -1111,7 +1099,7 @@ nautilus_tree_view_destroy (GtkObject *object)
 	}
 
 	if (view->details->tree != NULL) {
-		gtk_object_unref (GTK_OBJECT (view->details->change_queue));
+		g_object_unref (view->details->change_queue);
 		
 		if (compare_cached_key == view->details->tree) {
 			compare_cached_key = NULL;
@@ -1144,20 +1132,32 @@ nautilus_tree_view_destroy (GtkObject *object)
 		nautilus_tree_view_free_dnd (view);
 		
 		disconnect_model_handlers (view);
-		gtk_object_unref (GTK_OBJECT (view->details->model));
+		g_object_unref (view->details->model);
 		
 		nautilus_tree_expansion_state_save (view->details->expansion_state);
-		gtk_object_unref (GTK_OBJECT (view->details->expansion_state));
+		g_object_unref (view->details->expansion_state);
+
+		view->details->tree = NULL;
 	}
 
 	eel_gtk_object_list_free (view->details->unparented_tree_nodes);
 	
+	EEL_CALL_PARENT (BONOBO_OBJECT_CLASS, destroy, (object));
+}
+
+static void
+nautilus_tree_view_finalize (GObject *object)
+{
+	NautilusTreeView *view;
+	
+	view = NAUTILUS_TREE_VIEW (object);
+
 	g_free (view->details->current_main_view_uri);
 	g_free (view->details->selected_uri);
 
 	g_free (view->details);
-	
-	EEL_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
+
+	EEL_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
 }
 
 static EelCTreeNode *
@@ -1372,7 +1372,7 @@ tree_load_location_callback (NautilusView *nautilus_view,
 			     NautilusTreeView *view)
 {
 	g_assert (nautilus_view == NAUTILUS_VIEW (view));
-	
+
 	nautilus_view_report_load_complete (nautilus_view);
 
 	g_free (view->details->current_main_view_uri);
@@ -1619,11 +1619,14 @@ nautilus_tree_view_update_all_icons (NautilusTreeView *view)
 }
 
 static void
-nautilus_tree_view_initialize_class (NautilusTreeViewClass *klass)
+nautilus_tree_view_class_init (NautilusTreeViewClass *klass)
 {
-	GtkObjectClass *object_class;
+	GObjectClass *gobject_class;
+	BonoboObjectClass *object_class;
 	
-	object_class = GTK_OBJECT_CLASS (klass);
+	gobject_class = G_OBJECT_CLASS (klass);
+	object_class = BONOBO_OBJECT_CLASS (klass);
 	
 	object_class->destroy = nautilus_tree_view_destroy;
+	gobject_class->finalize = nautilus_tree_view_finalize;
 }
