@@ -19,7 +19,7 @@
    Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
   
-   Author: Darin Adler <darin@eazel.com>
+   Author: Darin Adler <darin@bentspoon.com>
 */
 
 #include "nautilus-directory-metafile-monitor.h"
@@ -27,14 +27,16 @@
 #include "nautilus-file.h"
 #include "nautilus-metafile-server.h"
 #include "nautilus-monitor.h"
+#include "nautilus-file-queue.h"
 #include <eel/eel-vfs-extensions.h>
-#include <gnome-xml/tree.h>
+#include <libxml/tree.h>
 #include <libgnomevfs/gnome-vfs-file-info.h>
 #include <libgnomevfs/gnome-vfs-types.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
 #include <libnautilus/nautilus-idle-queue.h>
 
-typedef struct ActivationURIReadState ActivationURIReadState;
+typedef struct LinkInfoReadState LinkInfoReadState;
 typedef struct TopLeftTextReadState TopLeftTextReadState;
 typedef struct FileMonitors FileMonitors;
 
@@ -49,6 +51,10 @@ struct NautilusDirectoryDetails
 	GList *file_list;
 	GHashTable *file_hash;
 
+	/* Queues of files needing some I/O done. */
+	NautilusFileQueue *high_priority_queue;
+	NautilusFileQueue *low_priority_queue;
+
 	/* These lists are going to be pretty short.  If we think they
 	 * are going to get big, we can use hash tables instead.
 	 */
@@ -75,7 +81,6 @@ struct NautilusDirectoryDetails
 
 	NautilusFile *load_directory_file;
 	int load_file_count;
-	GnomeVFSDirectoryFilter *load_file_count_filter;
 	GHashTable *load_mime_list_hash;
 
 	GList *get_file_infos_in_progress; /* list of GnomeVFSAsyncHandle * */
@@ -96,14 +101,15 @@ struct NautilusDirectoryDetails
 	GnomeVFSAsyncHandle *get_info_in_progress;
 
 	TopLeftTextReadState *top_left_read_state;
-	ActivationURIReadState *activation_uri_read_state;
+
+	LinkInfoReadState *link_info_read_state;
 
 	GList *file_operations_in_progress; /* list of FileOperation * */
 };
 
 /* A request for information about one or more files. */
 typedef struct {
-	gboolean activation_uri;
+	gboolean link_info;
 	gboolean deep_count;
 	gboolean directory_count;
 	gboolean file_info;
@@ -137,7 +143,9 @@ void               nautilus_directory_monitor_add_internal            (NautilusD
 								       gconstpointer              client,
 								       gboolean                   monitor_hidden_files,
 								       gboolean                   monitor_backup_files,
-								       GList                     *attributes);
+								       GList                     *attributes,
+								       NautilusDirectoryCallback  callback,
+								       gpointer                   callback_data);
 void               nautilus_directory_monitor_remove_internal         (NautilusDirectory         *directory,
 								       NautilusFile              *file,
 								       gconstpointer              client);
@@ -202,6 +210,12 @@ void               nautilus_directory_end_file_name_change            (NautilusD
 								       GList                     *node);
 void               nautilus_directory_moved                           (const char                *from_uri,
 								       const char                *to_uri);
+/* Interface to the work queue. */
+
+void               nautilus_directory_add_file_to_work_queue          (NautilusDirectory *directory,
+								       NautilusFile *file);
+void               nautilus_directory_remove_file_from_work_queue     (NautilusDirectory *directory,
+								       NautilusFile *file);
 
 /* debugging functions */
 int                nautilus_directory_number_outstanding              (void);
