@@ -19,7 +19,7 @@
    Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
   
-   Author: Darin Adler <darin@eazel.com>
+   Author: Darin Adler <darin@bentspoon.com>
 */
 
 #include <config.h>
@@ -27,6 +27,7 @@
 
 #include "nautilus-directory-metafile.h"
 #include "nautilus-directory-notify.h"
+#include "nautilus-file-attributes.h"
 #include "nautilus-file-private.h"
 #include "nautilus-file-utilities.h"
 #include "nautilus-global-preferences.h"
@@ -35,7 +36,6 @@
 #include "nautilus-metafile.h"
 #include "nautilus-trash-directory.h"
 #include "nautilus-vfs-directory.h"
-#include <ctype.h>
 #include <eel/eel-glib-extensions.h>
 #include <eel/eel-gtk-macros.h>
 #include <eel/eel-string.h>
@@ -62,64 +62,66 @@ static guint signals[LAST_SIGNAL];
 
 static GHashTable *directories;
 
-static void               nautilus_directory_destroy          (GtkObject              *object);
-static void               nautilus_directory_initialize       (gpointer                object,
+static void               nautilus_directory_finalize         (GObject                *object);
+static void               nautilus_directory_init             (gpointer                object,
 							       gpointer                klass);
-static void               nautilus_directory_initialize_class (NautilusDirectoryClass *klass);
+static void               nautilus_directory_class_init (NautilusDirectoryClass *klass);
 static NautilusDirectory *nautilus_directory_new              (const char             *uri);
 static char *             real_get_name_for_self_as_new_file  (NautilusDirectory      *directory);
 static void               set_directory_uri                   (NautilusDirectory      *directory,
 							       const char             *new_uri);
 
-EEL_DEFINE_CLASS_BOILERPLATE (NautilusDirectory,
-				   nautilus_directory,
-				   GTK_TYPE_OBJECT)
+EEL_CLASS_BOILERPLATE (NautilusDirectory,
+		       nautilus_directory,
+		       GTK_TYPE_OBJECT)
 
 static void
-nautilus_directory_initialize_class (NautilusDirectoryClass *klass)
+nautilus_directory_class_init (NautilusDirectoryClass *klass)
 {
-	GtkObjectClass *object_class;
+	GObjectClass *object_class;
 
-	object_class = GTK_OBJECT_CLASS (klass);
+	object_class = G_OBJECT_CLASS (klass);
 	
-	object_class->destroy = nautilus_directory_destroy;
+	object_class->finalize = nautilus_directory_finalize;
 
 	signals[FILES_ADDED] =
-		gtk_signal_new ("files_added",
-				GTK_RUN_LAST,
-				object_class->type,
-				GTK_SIGNAL_OFFSET (NautilusDirectoryClass, files_added),
-				gtk_marshal_NONE__POINTER,
-				GTK_TYPE_NONE, 1, GTK_TYPE_POINTER);
+		g_signal_new ("files_added",
+		              G_TYPE_FROM_CLASS (object_class),
+		              G_SIGNAL_RUN_LAST,
+		              G_STRUCT_OFFSET (NautilusDirectoryClass, files_added),
+		              NULL, NULL,
+		              gtk_marshal_VOID__POINTER,
+		              G_TYPE_NONE, 1, G_TYPE_POINTER);
 	signals[FILES_CHANGED] =
-		gtk_signal_new ("files_changed",
-				GTK_RUN_LAST,
-				object_class->type,
-				GTK_SIGNAL_OFFSET (NautilusDirectoryClass, files_changed),
-				gtk_marshal_NONE__POINTER,
-				GTK_TYPE_NONE, 1, GTK_TYPE_POINTER);
+		g_signal_new ("files_changed",
+		              G_TYPE_FROM_CLASS (object_class),
+		              G_SIGNAL_RUN_LAST,
+		              G_STRUCT_OFFSET (NautilusDirectoryClass, files_changed),
+		              NULL, NULL,
+		              gtk_marshal_VOID__POINTER,
+		              G_TYPE_NONE, 1, G_TYPE_POINTER);
 	signals[DONE_LOADING] =
-		gtk_signal_new ("done_loading",
-				GTK_RUN_LAST,
-				object_class->type,
-				GTK_SIGNAL_OFFSET (NautilusDirectoryClass, done_loading),
-				gtk_marshal_NONE__NONE,
-				GTK_TYPE_NONE, 0);
+		g_signal_new ("done_loading",
+		              G_TYPE_FROM_CLASS (object_class),
+		              G_SIGNAL_RUN_LAST,
+		              G_STRUCT_OFFSET (NautilusDirectoryClass, done_loading),
+		              NULL, NULL,
+		              gtk_marshal_VOID__VOID,
+		              G_TYPE_NONE, 0);
 	signals[LOAD_ERROR] =
-		gtk_signal_new ("load_error",
-				GTK_RUN_LAST,
-				object_class->type,
-				GTK_SIGNAL_OFFSET (NautilusDirectoryClass, load_error),
-				gtk_marshal_NONE__INT,
-				GTK_TYPE_NONE, 1, GTK_TYPE_INT);
-	
-	gtk_object_class_add_signals (object_class, signals, LAST_SIGNAL);
+		g_signal_new ("load_error",
+		              G_TYPE_FROM_CLASS (object_class),
+		              G_SIGNAL_RUN_LAST,
+		              G_STRUCT_OFFSET (NautilusDirectoryClass, load_error),
+		              NULL, NULL,
+		              gtk_marshal_VOID__INT,
+		              G_TYPE_NONE, 1, G_TYPE_INT);
 
 	klass->get_name_for_self_as_new_file = real_get_name_for_self_as_new_file;
 }
 
 static void
-nautilus_directory_initialize (gpointer object, gpointer klass)
+nautilus_directory_init (gpointer object, gpointer klass)
 {
 	NautilusDirectory *directory;
 
@@ -127,6 +129,8 @@ nautilus_directory_initialize (gpointer object, gpointer klass)
 
 	directory->details = g_new0 (NautilusDirectoryDetails, 1);
 	directory->details->file_hash = g_hash_table_new (g_str_hash, g_str_equal);
+	directory->details->high_priority_queue = nautilus_file_queue_new ();
+	directory->details->low_priority_queue = nautilus_file_queue_new ();
 	directory->details->idle_queue = nautilus_idle_queue_new ();
 }
 
@@ -139,7 +143,7 @@ nautilus_directory_ref (NautilusDirectory *directory)
 
 	g_return_if_fail (NAUTILUS_IS_DIRECTORY (directory));
 
-	gtk_object_ref (GTK_OBJECT (directory));
+	g_object_ref (directory);
 }
 
 void
@@ -151,16 +155,18 @@ nautilus_directory_unref (NautilusDirectory *directory)
 
 	g_return_if_fail (NAUTILUS_IS_DIRECTORY (directory));
 
-	gtk_object_unref (GTK_OBJECT (directory));
+	g_object_unref (directory);
 }
 
 static void
-nautilus_directory_destroy (GtkObject *object)
+nautilus_directory_finalize (GObject *object)
 {
 	NautilusDirectory *directory;
 
 	directory = NAUTILUS_DIRECTORY (object);
 
+	g_hash_table_remove (directories, directory->details->uri);
+
 	nautilus_directory_cancel (directory);
 	g_assert (directory->details->count_in_progress == NULL);
 	g_assert (directory->details->top_left_read_state == NULL);
@@ -178,22 +184,23 @@ nautilus_directory_destroy (GtkObject *object)
 		nautilus_directory_unregister_metadata_monitor (directory);
 	}
 
-	if (directory->details->metafile_corba_object != CORBA_OBJECT_NIL) {
-		bonobo_object_release_unref (directory->details->metafile_corba_object, NULL);
-	}
-
-	g_hash_table_remove (directories, directory->details->uri);
+	bonobo_object_release_unref (directory->details->metafile_corba_object, NULL);
 
 	if (directory->details->dequeue_pending_idle_id != 0) {
 		gtk_idle_remove (directory->details->dequeue_pending_idle_id);
 	}
  
 	g_free (directory->details->uri);
+
 	if (directory->details->vfs_uri != NULL) {
 		gnome_vfs_uri_unref (directory->details->vfs_uri);
 	}
+
 	g_assert (directory->details->file_list == NULL);
 	g_hash_table_destroy (directory->details->file_hash);
+
+	nautilus_file_queue_destroy (directory->details->high_priority_queue);
+	nautilus_file_queue_destroy (directory->details->low_priority_queue);
 	nautilus_idle_queue_destroy (directory->details->idle_queue);
 	g_assert (directory->details->directory_load_in_progress == NULL);
 	g_assert (directory->details->count_in_progress == NULL);
@@ -202,7 +209,7 @@ nautilus_directory_destroy (GtkObject *object)
 
 	g_free (directory->details);
 
-	EEL_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
+	EEL_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
 }
 
 static void
@@ -235,7 +242,7 @@ emit_change_signals_for_all_files (NautilusDirectory *directory)
 {
 	GList *files;
 
-	files = eel_g_list_copy (directory->details->file_list);
+	files = g_list_copy (directory->details->file_list);
 	if (directory->details->as_file != NULL) {
 		files = g_list_prepend (files, directory->details->as_file);
 	}
@@ -305,10 +312,11 @@ nautilus_directory_make_uri_canonical (const char *uri)
 	/* To NautilusDirectory, a uri with or without a trailing
 	 * / is equivalent. This is necessary to prevent separate
 	 * NautilusDirectories for the same location from being
-	 * created. (See bugzilla.eazel.com 3322 for an example.)
+	 * created. (See bugzilla.gnome.org 43322 for an example.)
 	 */
 	canonical = eel_str_strip_trailing_chr (canonical_maybe_trailing_slash, '/');
-	if (strcmp (canonical, canonical_maybe_trailing_slash) != 0) {
+	if (strcmp (canonical, canonical_maybe_trailing_slash) != 0 &&
+	    strcmp (canonical, "favorites:") != 0) {
 		/* If some trailing '/' were stripped, there's the possibility,
 		 * that we stripped away all the '/' from a uri that has only
 		 * '/' characters. If you change this code, check to make sure
@@ -478,11 +486,11 @@ nautilus_directory_new (const char *uri)
 	g_assert (uri != NULL);
 
 	if (eel_uri_is_trash (uri)) {
-		directory = NAUTILUS_DIRECTORY (gtk_object_new (NAUTILUS_TYPE_TRASH_DIRECTORY, NULL));
+		directory = NAUTILUS_DIRECTORY (g_object_new (NAUTILUS_TYPE_TRASH_DIRECTORY, NULL));
 	} else {
-		directory = NAUTILUS_DIRECTORY (gtk_object_new (NAUTILUS_TYPE_VFS_DIRECTORY, NULL));
+		directory = NAUTILUS_DIRECTORY (g_object_new (NAUTILUS_TYPE_VFS_DIRECTORY, NULL));
 	}
-	gtk_object_ref (GTK_OBJECT (directory));
+	g_object_ref (directory);
 	gtk_object_sink (GTK_OBJECT (directory));
 
 	set_directory_uri (directory, uri);
@@ -560,6 +568,7 @@ nautilus_directory_add_file (NautilusDirectory *directory, NautilusFile *file)
 	/* Ref if we are monitoring. */
 	if (nautilus_directory_is_file_list_monitored (directory)) {
 		nautilus_file_ref (file);
+		nautilus_directory_add_file_to_work_queue (directory, file);
 	}
 }
 
@@ -582,6 +591,8 @@ nautilus_directory_remove_file (NautilusDirectory *directory, NautilusFile *file
 		(directory->details->file_list, node);
 	g_list_free_1 (node);
 
+	nautilus_directory_remove_file_from_work_queue (directory, file);
+
 	if (!file->details->unconfirmed) {
 		directory->details->confirmed_file_count--;
 	}
@@ -658,7 +669,6 @@ nautilus_directory_find_file_by_internal_uri (NautilusDirectory *directory,
 		result = nautilus_directory_get_existing_corresponding_file (directory);
 		if (result != NULL) {
 			nautilus_file_unref (result);
-			g_return_val_if_fail (!GTK_OBJECT_DESTROYED (result), NULL);
 		}
 	} else {
 		result = nautilus_directory_find_file_by_relative_uri (directory, relative_uri);
@@ -672,8 +682,8 @@ nautilus_directory_emit_files_added (NautilusDirectory *directory,
 				     GList *added_files)
 {
 	if (added_files != NULL) {
-		gtk_signal_emit (GTK_OBJECT (directory),
-				 signals[FILES_ADDED],
+		g_signal_emit (directory,
+				 signals[FILES_ADDED], 0,
 				 added_files);
 	}
 }
@@ -683,8 +693,8 @@ nautilus_directory_emit_files_changed (NautilusDirectory *directory,
 				       GList *changed_files)
 {
 	if (changed_files != NULL) {
-		gtk_signal_emit (GTK_OBJECT (directory),
-				 signals[FILES_CHANGED],
+		g_signal_emit (directory,
+				 signals[FILES_CHANGED], 0,
 				 changed_files);
 	}
 }
@@ -704,16 +714,16 @@ nautilus_directory_emit_change_signals (NautilusDirectory *directory,
 void
 nautilus_directory_emit_done_loading (NautilusDirectory *directory)
 {
-	gtk_signal_emit (GTK_OBJECT (directory),
-			 signals[DONE_LOADING]);
+	g_signal_emit (directory,
+			 signals[DONE_LOADING], 0);
 }
 
 void
 nautilus_directory_emit_load_error (NautilusDirectory *directory,
 				    GnomeVFSResult error_result)
 {
-	gtk_signal_emit (GTK_OBJECT (directory),
-			 signals[LOAD_ERROR],
+	g_signal_emit (directory,
+			 signals[LOAD_ERROR], 0,
 			 error_result);
 }
 
@@ -791,8 +801,8 @@ call_files_added_free_list (gpointer key, gpointer value, gpointer user_data)
 	g_assert (value != NULL);
 	g_assert (user_data == NULL);
 
-	gtk_signal_emit (GTK_OBJECT (key),
-			 signals[FILES_ADDED],
+	g_signal_emit (key,
+			 signals[FILES_ADDED], 0,
 			 value);
 	g_list_free (value);
 }
@@ -862,7 +872,9 @@ nautilus_directory_notify_files_added (GList *uris)
 	NautilusDirectory *directory;
 	GHashTable *parent_directories;
 	const char *uri;
+	const char *directory_uri;
 	GnomeVFSURI *vfs_uri;
+	NautilusFile *file;
 
 	/* Make a list of added files in each directory. */
 	added_lists = g_hash_table_new (NULL, NULL);
@@ -876,6 +888,19 @@ nautilus_directory_notify_files_added (GList *uris)
 		/* See if the directory is already known. */
 		directory = get_parent_directory_if_exists (uri);
 		if (directory == NULL) {
+			/* In case the directory is not being
+			 * monitored, but the corresponding file is,
+			 * we must invalidate it's item count.
+			 */
+			
+			directory_uri = uri_get_directory_part (uri);
+			file = nautilus_file_get_existing (directory_uri);
+
+			if (file != NULL) {
+				nautilus_file_invalidate_count_and_mime_list (file);
+				nautilus_file_unref (file);
+			}
+
 			continue;
 		}
 
@@ -1412,6 +1437,7 @@ nautilus_directory_get_file_uri (NautilusDirectory *directory,
 void
 nautilus_directory_call_when_ready (NautilusDirectory *directory,
 				    GList *file_attributes,
+				    gboolean wait_for_all_files,
 				    NautilusDirectoryCallback callback,
 				    gpointer callback_data)
 {
@@ -1420,11 +1446,10 @@ nautilus_directory_call_when_ready (NautilusDirectory *directory,
 
 	EEL_CALL_METHOD
 		(NAUTILUS_DIRECTORY_CLASS, directory,
-		 call_when_ready, (directory, file_attributes,
+		 call_when_ready, (directory, file_attributes, wait_for_all_files,
 				   callback, callback_data));
 }
 
-
 void
 nautilus_directory_cancel_callback (NautilusDirectory *directory,
 				    NautilusDirectoryCallback callback,
@@ -1443,7 +1468,9 @@ nautilus_directory_file_monitor_add (NautilusDirectory *directory,
 				     gconstpointer client,
 				     gboolean monitor_hidden_files,
 				     gboolean monitor_backup_files,
-				     GList *file_attributes)
+				     GList *file_attributes,
+				     NautilusDirectoryCallback callback,
+				     gpointer callback_data)
 {
 	g_return_if_fail (NAUTILUS_IS_DIRECTORY (directory));
 	g_return_if_fail (client != NULL);
@@ -1453,7 +1480,8 @@ nautilus_directory_file_monitor_add (NautilusDirectory *directory,
 		 file_monitor_add, (directory, client,
 				    monitor_hidden_files,
 				    monitor_backup_files,
-				    file_attributes));
+				    file_attributes,
+				    callback, callback_data));
 }
 
 void
@@ -1488,6 +1516,105 @@ nautilus_directory_is_not_empty (NautilusDirectory *directory)
 		 is_not_empty, (directory));
 }
 
+static gboolean
+is_tentative (gpointer data, gpointer callback_data)
+{
+	NautilusFile *file;
+
+	g_assert (callback_data == NULL);
+
+	file = NAUTILUS_FILE (data);
+	return file->details->info == NULL;
+}
+
+GList *
+nautilus_directory_get_file_list (NautilusDirectory *directory)
+{
+	GList *tentative_files, *non_tentative_files;
+
+	tentative_files = eel_g_list_partition
+		(g_list_copy (directory->details->file_list),
+		 is_tentative, NULL, &non_tentative_files);
+	g_list_free (tentative_files);
+
+	nautilus_file_list_ref (non_tentative_files);
+	return non_tentative_files;
+}
+
+/**
+ * nautilus_directory_list_ref
+ *
+ * Ref all the directories in a list.
+ * @list: GList of directories.
+ **/
+GList *
+nautilus_directory_list_ref (GList *list)
+{
+	g_list_foreach (list, (GFunc) nautilus_directory_ref, NULL);
+	return list;
+}
+
+/**
+ * nautilus_directory_list_unref
+ *
+ * Unref all the directories in a list.
+ * @list: GList of directories.
+ **/
+void
+nautilus_directory_list_unref (GList *list)
+{
+	g_list_foreach (list, (GFunc) nautilus_directory_unref, NULL);
+}
+
+/**
+ * nautilus_directory_list_free
+ *
+ * Free a list of directories after unrefing them.
+ * @list: GList of directories.
+ **/
+void
+nautilus_directory_list_free (GList *list)
+{
+	nautilus_directory_list_unref (list);
+	g_list_free (list);
+}
+
+/**
+ * nautilus_directory_list_copy
+ *
+ * Copy the list of directories, making a new ref of each,
+ * @list: GList of directories.
+ **/
+GList *
+nautilus_directory_list_copy (GList *list)
+{
+	return g_list_copy (nautilus_directory_list_ref (list));
+}
+
+static int
+compare_by_uri (NautilusDirectory *a, NautilusDirectory *b)
+{
+	return strcmp (a->details->uri, b->details->uri);
+}
+
+static int
+compare_by_uri_cover (gconstpointer a, gconstpointer b)
+{
+	return compare_by_uri (NAUTILUS_DIRECTORY (a), NAUTILUS_DIRECTORY (b));
+}
+
+/**
+ * nautilus_directory_list_sort_by_uri
+ * 
+ * Sort the list of directories by directory uri.
+ * @list: GList of directories.
+ **/
+GList *
+nautilus_directory_list_sort_by_uri (GList *list)
+{
+	return g_list_sort (list, compare_by_uri_cover);
+}
+
 #if !defined (NAUTILUS_OMIT_SELF_CHECK)
 
 #include <eel/eel-debug.h>
@@ -1537,12 +1664,12 @@ nautilus_self_check_directory (void)
 
 	nautilus_directory_file_monitor_add
 		(directory, &data_dummy,
-		 TRUE, TRUE, NULL);
+		 TRUE, TRUE, NULL, NULL, NULL);
 
 	got_metadata_flag = FALSE;
 
-	attributes = g_list_append (NULL, NAUTILUS_FILE_ATTRIBUTE_METADATA);
-	nautilus_directory_call_when_ready (directory, attributes,
+	attributes = g_list_prepend (NULL, NAUTILUS_FILE_ATTRIBUTE_METADATA);
+	nautilus_directory_call_when_ready (directory, attributes, TRUE,
 					    got_metadata_callback, &data_dummy);
 	g_list_free (attributes);
 
@@ -1592,8 +1719,8 @@ nautilus_self_check_directory (void)
 	directory = nautilus_directory_get ("file:///etc");
 
 	got_metadata_flag = FALSE;
-	attributes = g_list_append (NULL, NAUTILUS_FILE_ATTRIBUTE_METADATA);
-	nautilus_directory_call_when_ready (directory, attributes,
+	attributes = g_list_prepend (NULL, NAUTILUS_FILE_ATTRIBUTE_METADATA);
+	nautilus_directory_call_when_ready (directory, attributes, TRUE,
 					    got_metadata_callback, &data_dummy);
 	g_list_free (attributes);
 
@@ -1607,7 +1734,7 @@ nautilus_self_check_directory (void)
 
 	attributes = g_list_prepend (NULL, NAUTILUS_FILE_ATTRIBUTE_MIME_TYPE);
 	attributes = g_list_prepend (attributes, NAUTILUS_FILE_ATTRIBUTE_DEEP_COUNTS);
-	nautilus_directory_call_when_ready (directory, attributes,
+	nautilus_directory_call_when_ready (directory, attributes, TRUE,
 					    got_files_callback, &data_dummy);
 	g_list_free (attributes);
 
@@ -1655,7 +1782,7 @@ nautilus_self_check_directory (void)
 	EEL_CHECK_STRING_RESULT (nautilus_directory_make_uri_canonical ("http://le-hackeur.org/"), "http://le-hackeur.org");
 	EEL_CHECK_STRING_RESULT (nautilus_directory_make_uri_canonical ("http://le-hackeur.org/dir"), "http://le-hackeur.org/dir");
 	EEL_CHECK_STRING_RESULT (nautilus_directory_make_uri_canonical ("http://le-hackeur.org/dir/"), "http://le-hackeur.org/dir");
-	/* FIXME bugzilla.eazel.com 5068: the "nested" URI loses some characters here. Maybe that's OK because we escape them in practice? */
+	/* FIXME bugzilla.gnome.org 45068: the "nested" URI loses some characters here. Maybe that's OK because we escape them in practice? */
 	EEL_CHECK_STRING_RESULT (nautilus_directory_make_uri_canonical ("search://[file://]file_name contains stuff"), "search://[file/]file_name contains stuff");
 	EEL_CHECK_STRING_RESULT (nautilus_directory_make_uri_canonical ("eazel-services:/~turtle"), "eazel-services:///~turtle");
 	EEL_CHECK_STRING_RESULT (nautilus_directory_make_uri_canonical ("eazel-services:///~turtle"), "eazel-services:///~turtle");
