@@ -19,7 +19,7 @@
    Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
   
-   Author: Darin Adler <darin@eazel.com>
+   Author: Darin Adler <darin@bentspoon.com>
 */
 
 #ifndef NAUTILUS_DIRECTORY_H
@@ -57,10 +57,10 @@
 typedef struct NautilusFile NautilusFile;
 #endif
 
-/* FIXME bugzilla.eazel.com 5382:
+/* FIXME bugzilla.gnome.org 45382:
  * Increase or remove this limit? 
  */
-/* FIXME bugzilla.eazel.com 5603:
+/* FIXME bugzilla.gnome.org 45603:
  * This limit is not actually "hard", which can lead to some minor UI problems.
  */
 #define NAUTILUS_DIRECTORY_FILE_LIST_HARD_LIMIT     4000
@@ -114,6 +114,7 @@ typedef struct
 					  NautilusFile              *file);
 	void     (* call_when_ready)     (NautilusDirectory         *directory,
 					  GList                     *file_attributes,
+					  gboolean                   wait_for_file_list,
 					  NautilusDirectoryCallback  callback,
 					  gpointer                   callback_data);
 	void     (* cancel_callback)     (NautilusDirectory         *directory,
@@ -123,7 +124,9 @@ typedef struct
 					  gconstpointer              client,
 					  gboolean                   monitor_hidden_files,
 					  gboolean                   monitor_backup_files,
-					  GList                     *monitor_attributes);
+					  GList                     *monitor_attributes,
+					  NautilusDirectoryCallback  initial_files_callback,
+					  gpointer                   callback_data);
 	void     (* file_monitor_remove) (NautilusDirectory         *directory,
 					  gconstpointer              client);
 	void     (* force_reload)        (NautilusDirectory         *directory);
@@ -157,20 +160,18 @@ gboolean           nautilus_directory_contains_file            (NautilusDirector
 								NautilusFile              *file);
 
 /* Get the uri of the file in the directory, NULL if not found */
-char *		   nautilus_directory_get_file_uri	       (NautilusDirectory	  *directory,
-								const char 		  *file_name);
+char *             nautilus_directory_get_file_uri             (NautilusDirectory         *directory,
+								const char                *file_name);
 
 /* Get (and ref) a NautilusFile object for this directory. */
 NautilusFile *     nautilus_directory_get_corresponding_file   (NautilusDirectory         *directory);
-							    
 
 /* Waiting for data that's read asynchronously.
  * The file attribute and metadata keys are for files in the directory.
- * If any file attributes or metadata keys are passed, it won't call
- * until all the files are seen.
  */
 void               nautilus_directory_call_when_ready          (NautilusDirectory         *directory,
 								GList                     *file_attributes,
+								gboolean                   wait_for_all_files,
 								NautilusDirectoryCallback  callback,
 								gpointer                   callback_data);
 void               nautilus_directory_cancel_callback          (NautilusDirectory         *directory,
@@ -183,11 +184,16 @@ void               nautilus_directory_file_monitor_add         (NautilusDirector
 								gconstpointer              client,
 								gboolean                   monitor_hidden_files,
 								gboolean                   monitor_backup_files,
-								GList                     *attributes);
+								GList                     *attributes,
+								NautilusDirectoryCallback  initial_files_callback,
+								gpointer                   callback_data);
 void               nautilus_directory_file_monitor_remove      (NautilusDirectory         *directory,
 								gconstpointer              client);
 void               nautilus_directory_force_reload             (NautilusDirectory         *directory);
 
+/* Get a list of all files currently known in the directory. */
+GList *            nautilus_directory_get_file_list            (NautilusDirectory         *directory);
+
 /* Return true if the directory has information about all the files.
  * This will be false until the directory has been read at least once.
  */
@@ -201,7 +207,15 @@ gboolean           nautilus_directory_is_local                 (NautilusDirector
  */
 gboolean           nautilus_directory_is_not_empty             (NautilusDirectory         *directory);
 gboolean           nautilus_directory_file_list_length_reached (NautilusDirectory         *directory);
+char *             nautilus_directory_make_uri_canonical       (const char                *uri);
 
-char *		   nautilus_directory_make_uri_canonical       (const char *uri);
+/* Convenience functions for dealing with a list of NautilusDirectory objects that each have a ref.
+ * These are just convenient names for functions that work on lists of GtkObject *.
+ */
+GList *            nautilus_directory_list_ref                 (GList                     *directory_list);
+void               nautilus_directory_list_unref               (GList                     *directory_list);
+void               nautilus_directory_list_free                (GList                     *directory_list);
+GList *            nautilus_directory_list_copy                (GList                     *directory_list);
+GList *            nautilus_directory_list_sort_by_uri         (GList                     *directory_list);
 
 #endif /* NAUTILUS_DIRECTORY_H */
