@@ -20,15 +20,14 @@
    write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
 
-   Authors: Ettore Perazzoli <ettore@gnu.org>, Darin Adler <darin@eazel.com>
+   Authors: Ettore Perazzoli <ettore@gnu.org>, Darin Adler <darin@bentspoon.com>
 */
 
 #ifndef NAUTILUS_ICON_CONTAINER_H
 #define NAUTILUS_ICON_CONTAINER_H
 
-#include <libgnomeui/gnome-canvas.h>
+#include <libgnomecanvas/gnome-canvas.h>
 #include "nautilus-icon-factory.h"
-#include <eel/eel-scalable-font.h>
 
 #define NAUTILUS_ICON_CONTAINER(obj) \
 	GTK_CHECK_CAST (obj, nautilus_icon_container_get_type (), NautilusIconContainer)
@@ -152,7 +151,7 @@ typedef struct {
 } NautilusIconContainerClass;
 
 /* GtkObject */
-guint             nautilus_icon_container_get_type                      (void);
+GType             nautilus_icon_container_get_type                      (void);
 GtkWidget *       nautilus_icon_container_new                           (void);
 
 
@@ -215,8 +214,6 @@ void              nautilus_icon_container_set_anti_aliased_mode         (Nautilu
 void              nautilus_icon_container_set_label_font_for_zoom_level (NautilusIconContainer  *container,
 									 int                     zoom_level,
 									 GdkFont                *font);
-void              nautilus_icon_container_set_smooth_label_font         (NautilusIconContainer  *container,
-									 EelScalableFont   *font);
 gboolean          nautilus_icon_container_get_is_fixed_size             (NautilusIconContainer  *container);
 void              nautilus_icon_container_set_is_fixed_size             (NautilusIconContainer  *container,
 									 gboolean                is_fixed_size);
