@@ -28,7 +28,6 @@
 #include "fm-desktop-icon-view.h"
 #include "fm-error-reporting.h"
 #include <bonobo/bonobo-ui-util.h>
-#include <ctype.h>
 #include <eel/eel-background.h>
 #include <eel/eel-glib-extensions.h>
 #include <eel/eel-gtk-extensions.h>
@@ -47,18 +46,16 @@
 #include <gtk/gtkwindow.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-config.h>
-#include <libgnome/gnome-mime.h>
+#include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-async-ops.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <libgnomevfs/gnome-vfs-xfer.h>
-#include <libgnomevfs/gnome-vfs-mime.h>
 #include <libnautilus-private/nautilus-audio-player.h>
 #include <libnautilus-private/nautilus-bonobo-extensions.h>
 #include <libnautilus-private/nautilus-directory-background.h>
 #include <libnautilus-private/nautilus-directory.h>
 #include <libnautilus-private/nautilus-file-utilities.h>
-#include <libnautilus-private/nautilus-font-factory.h>
 #include <libnautilus-private/nautilus-global-preferences.h>
 #include <libnautilus-private/nautilus-icon-container.h>
 #include <libnautilus-private/nautilus-icon-factory.h>
@@ -75,6 +72,7 @@
 #include <unistd.h>
 
 #define USE_OLD_AUDIO_PREVIEW 1
+#define READ_CHUNK_SIZE 16384
 
 /* Paths to use when creating & referring to Bonobo menu items */
 #define MENU_PATH_RENAME 			"/menu/File/File Items Placeholder/Rename"
@@ -120,8 +118,8 @@ typedef enum {
 
 /* forward declarations */
 static void                 create_icon_container                              (FMIconView            *icon_view);
-static void                 fm_icon_view_initialize                            (FMIconView            *icon_view);
-static void                 fm_icon_view_initialize_class                      (FMIconViewClass       *klass);
+static void                 fm_icon_view_init                            (FMIconView            *icon_view);
+static void                 fm_icon_view_class_init                      (FMIconViewClass       *klass);
 static void                 fm_icon_view_set_directory_sort_by                 (FMIconView            *icon_view,
 										NautilusFile          *file,
 										const char            *sort_by);
@@ -130,7 +128,6 @@ static void                 fm_icon_view_set_zoom_level                        (
 										gboolean               always_set_level);
 gboolean                    fm_icon_view_supports_auto_layout                  (FMIconView            *view);
 static void                 fm_icon_view_update_icon_container_fonts           (FMIconView            *icon_view);
-static void                 fm_icon_view_update_icon_container_smooth_font     (FMIconView            *icon_view);
 static void                 fm_icon_view_update_icon_container_font_size_table (FMIconView            *icon_view);
 static void                 fm_icon_view_update_click_mode                     (FMIconView            *icon_view);
 static void                 fm_icon_view_update_smooth_graphics_mode           (FMIconView            *icon_view);
@@ -159,7 +156,6 @@ static void                 default_use_manual_layout_changed_callback         (
 static void                 default_zoom_level_changed_callback                (gpointer               callback_data);
 static void                 default_zoom_level_font_size_changed_callback      (gpointer               callback_data);
 static void                 font_changed_callback                              (gpointer               callback_data);
-static void                 smooth_font_changed_callback                       (gpointer               callback_data);
 static void                 icon_view_handle_uri_list                          (NautilusIconContainer *container,
 										const char            *item_uris,
 										GdkDragAction          action,
@@ -170,7 +166,7 @@ static void                 icon_view_handle_uri_list                          (
 
 static int preview_sound_auto_value;
 
-EEL_DEFINE_CLASS_BOILERPLATE (FMIconView,
+EEL_CLASS_BOILERPLATE (FMIconView,
 			      fm_icon_view,
 			      FM_TYPE_DIRECTORY_VIEW)
 
@@ -236,7 +232,7 @@ struct FMIconViewDetails
 };
 
 static void
-fm_icon_view_destroy (GtkObject *object)
+fm_icon_view_finalize (GObject *object)
 {
 	FMIconView *icon_view;
 
@@ -246,7 +242,7 @@ fm_icon_view_destroy (GtkObject *object)
 	icon_view->details->menus_ready = FALSE;
 
 	if (icon_view->details->ui != NULL) {
-		bonobo_ui_component_unset_container (icon_view->details->ui);
+		bonobo_ui_component_unset_container (icon_view->details->ui, NULL);
 		bonobo_object_unref (BONOBO_OBJECT (icon_view->details->ui));
 	}
 
@@ -260,7 +256,7 @@ fm_icon_view_destroy (GtkObject *object)
 	nautilus_file_list_free (icon_view->details->icons_not_positioned);
 	g_free (icon_view->details);
 
-	EEL_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
+	EEL_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
 }
 
 static NautilusIconContainer *
@@ -867,7 +863,7 @@ get_sort_criterion_by_metadata_text (const char *metadata_text)
 	guint i;
 
 	/* Figure out what the new sort setting should be. */
-	for (i = 0; i < EEL_N_ELEMENTS (sort_criteria); i++) {
+	for (i = 0; i < G_N_ELEMENTS (sort_criteria); i++) {
 		if (strcmp (sort_criteria[i].metadata_text, metadata_text) == 0) {
 			return &sort_criteria[i];
 		}
@@ -881,7 +877,7 @@ get_sort_criterion_by_id (const char *id)
 	guint i;
 
 	/* Figure out what the new sort setting should be. */
-	for (i = 0; i < EEL_N_ELEMENTS (sort_criteria); i++) {
+	for (i = 0; i < G_N_ELEMENTS (sort_criteria); i++) {
 		if (strcmp (sort_criteria[i].id, id) == 0) {
 			return &sort_criteria[i];
 		}
@@ -895,7 +891,7 @@ get_sort_criterion_by_sort_type (NautilusFileSortType sort_type)
 	guint i;
 
 	/* Figure out what the new sort setting should be. */
-	for (i = 0; i < EEL_N_ELEMENTS (sort_criteria); i++) {
+	for (i = 0; i < G_N_ELEMENTS (sort_criteria); i++) {
 		if (sort_type == sort_criteria[i].sort_type) {
 			return &sort_criteria[i];
 		}
@@ -1313,6 +1309,7 @@ static void
 fm_icon_view_merge_menus (FMDirectoryView *view)
 {
 	FMIconView *icon_view;
+	Bonobo_UIContainer ui_container;
 	BonoboUIVerb verbs [] = {
 		BONOBO_UI_VERB ("Rename", rename_icon_callback),
 		BONOBO_UI_VERB ("Stretch", show_stretch_handles_callback),
@@ -1328,14 +1325,16 @@ fm_icon_view_merge_menus (FMDirectoryView *view)
 	icon_view = FM_ICON_VIEW (view);
 
 	icon_view->details->ui = bonobo_ui_component_new ("Icon View");
-	gtk_signal_connect (GTK_OBJECT (icon_view->details->ui),
-			    "ui_event", handle_ui_event, icon_view);
+	g_signal_connect (icon_view->details->ui,
+			    "ui_event", G_CALLBACK (handle_ui_event), icon_view);
+	ui_container = fm_directory_view_get_bonobo_ui_container (view);
 	bonobo_ui_component_set_container (icon_view->details->ui,
-					   fm_directory_view_get_bonobo_ui_container (view));
+					   ui_container, NULL);
+	bonobo_object_release_unref (ui_container, NULL);
 	bonobo_ui_util_set_ui (icon_view->details->ui,
 			       DATADIR,
 			       "nautilus-icon-view-ui.xml",
-			       "nautilus");
+			       "nautilus", NULL);
 
 	bonobo_ui_component_add_verb_list_with_data (icon_view->details->ui, verbs, view);
 	
@@ -1521,50 +1520,102 @@ band_select_ended_callback (NautilusIconContainer *container,
 
 /* handle the preview signal by inspecting the mime type.  For now, we only preview local sound files. */
 
-/* here's the timer task that actually plays the file using mpg123. */
+/* here's the timer task that actually plays the file using mpg123, ogg123 or play. */
 /* FIXME bugzilla.gnome.org 41258: we should get the application from our mime-type stuff */
 static gboolean
 play_file (gpointer callback_data)
 {
 #if USE_OLD_AUDIO_PREVIEW	
 	NautilusFile *file;
+	FMIconView *icon_view;
+	FILE *sound_process;
 	char *file_uri;
-	char *file_path, *mime_type;
+	char *suffix;
+	char *mime_type;
+	const char *command_str;
 	gboolean is_mp3;
 	gboolean is_ogg;
 	pid_t mp3_pid;
 	
-	file = NAUTILUS_FILE (callback_data);
+	GnomeVFSResult result;
+	GnomeVFSHandle *handle;
+	char *buffer;
+	GnomeVFSFileSize bytes_read;
+
+	icon_view = FM_ICON_VIEW (callback_data);
+	
+	file = icon_view->details->audio_preview_file;
 	file_uri = nautilus_file_get_uri (file);
-	file_path = gnome_vfs_get_local_path_from_uri (file_uri);
 	mime_type = nautilus_file_get_mime_type (file);
 	is_mp3 = eel_strcasecmp (mime_type, "audio/x-mp3") == 0;
 	is_ogg = eel_strcasecmp (mime_type, "application/x-ogg") == 0;
 
-	if (file_path != NULL) {
-		mp3_pid = fork ();
-		if (mp3_pid == (pid_t) 0) {
-			/* Set the group (session) id to this process for future killing. */
-			setsid();
-			if (is_mp3) {
-				execlp ("mpg123", "mpg123", "-y", "-q", file_path, NULL);
-			} else if (is_ogg) {
-				execlp ("ogg123", "ogg123", "-q", file_path, NULL);
-			} else {
-				execlp ("play", "play", file_path, NULL);
-			}
-			
-			_exit (0);
+	mp3_pid = fork ();
+	if (mp3_pid == (pid_t) 0) {
+		/* Set the group (session) id to this process for future killing. */
+		setsid();
+		if (is_mp3) {
+			command_str = "mpg123 -y -q -";
+		} else if (is_ogg) {
+			command_str = "ogg123 -q -";
 		} else {
-			nautilus_sound_register_sound (mp3_pid);
+			suffix = strrchr(file_uri, '.');
+			if (suffix == NULL) {
+				suffix = "wav";
+			} else {
+				suffix += 1; /* skip the period */
+			}
+			command_str = g_strdup_printf("play -t %s -", suffix);
+		}
+			
+		/* since the uri could be local or remote, we launch the sound player with popen and feed it
+		 * the data by fetching it with gnome_vfs
+		 */
+		sound_process = popen(command_str, "w");
+		if (sound_process == 0) {
+			return FALSE;
+		}
+			
+		/* read the file with gnome-vfs, feeding it to the sound player's standard input */
+		/* First, open the file. */
+		result = gnome_vfs_open (&handle, file_uri, GNOME_VFS_OPEN_READ);
+		if (result != GNOME_VFS_OK) {
+			return FALSE;
 		}
-	}
 
-	g_free (file_path);
+		/* allocate a buffer. */
+		buffer = g_malloc(READ_CHUNK_SIZE);
+			
+		/* read and write a chunk at a time, until we're done */
+		do {
+			result = gnome_vfs_read (handle,
+					buffer,
+					READ_CHUNK_SIZE,
+					&bytes_read);
+			if (result != GNOME_VFS_OK && result != GNOME_VFS_ERROR_EOF) {
+				g_free (buffer);
+				gnome_vfs_close (handle);
+				break;
+			}
+
+			/* pass the data the buffer to the sound process by writing to it */
+			fwrite(buffer, 1, bytes_read, sound_process);
+
+		} while (result == GNOME_VFS_OK);
+
+		/* Close the file. */
+		result = gnome_vfs_close (handle);			
+		g_free(buffer);
+		pclose(sound_process);
+		_exit (0);
+	} else {
+		nautilus_sound_register_sound (mp3_pid);
+	}
+		
 	g_free (file_uri);
 	g_free (mime_type);
 
-	/* FIXME: Need to zero out the timeout. */
+	icon_view->details->audio_preview_timeout = 0;
 #else
 	char *file_path, *file_uri, *mime_type;
 	gboolean is_mp3;
@@ -1621,7 +1672,7 @@ preview_audio (FMIconView *icon_view, NautilusFile *file, gboolean start_flag)
 	if (start_flag) {
 		icon_view->details->audio_preview_file = file;
 #if USE_OLD_AUDIO_PREVIEW			
-		icon_view->details->audio_preview_timeout = gtk_timeout_add (1000, play_file, file);
+		icon_view->details->audio_preview_timeout = gtk_timeout_add (1000, play_file, icon_view);
 #else
 		/* FIXME: Need to kill the existing timeout if there is one? */
 		icon_view->details->audio_preview_timeout = gtk_timeout_add (1000, play_file, icon_view);
@@ -1642,12 +1693,9 @@ should_preview_sound (NautilusFile *file) {
 		return FALSE;
 	}
 		
-	/* the following is disabled until we can preview remote sounds, which we currently can't do */
-	/*
-	if (preview_mode == NAUTILUS_SPEED_TRADEOFF_ALWAYS) {
+	if (preview_sound_auto_value == NAUTILUS_SPEED_TRADEOFF_ALWAYS) {
 		return TRUE;
 	}
-	*/
 	
 	return nautilus_file_is_local (file);
 }
@@ -1741,7 +1789,7 @@ fm_icon_view_sort_files (FMDirectoryView *view, GList **files)
 	if (!fm_icon_view_using_auto_layout (icon_view)) {
 		return;
 	}
-	*files = eel_g_list_sort_custom (*files, compare_files_cover, icon_view);
+	*files = g_list_sort_with_data (*files, compare_files_cover, icon_view);
 }
 
 static int
@@ -2018,6 +2066,7 @@ get_icon_text_callback (NautilusIconContainer *container,
  		*additional_text = NULL;
 		if (path != NULL) {
 			description = nautilus_link_local_get_additional_text (path);
+			g_free (path);
 			if (description)
 				*additional_text = g_strdup_printf (" \n%s\n ", description);
 			g_free (description);
@@ -2095,14 +2144,6 @@ font_changed_callback (gpointer callback_data)
 	fm_icon_view_update_icon_container_fonts (FM_ICON_VIEW (callback_data));
 }
 
-static void
-smooth_font_changed_callback (gpointer callback_data)
-{
- 	g_return_if_fail (FM_IS_ICON_VIEW (callback_data));
-
-	fm_icon_view_update_icon_container_smooth_font (FM_ICON_VIEW (callback_data));
-}
-
 static void
 default_zoom_level_font_size_changed_callback (gpointer callback_data)
 {
@@ -2127,14 +2168,6 @@ fm_icon_view_smooth_graphics_mode_changed (FMDirectoryView *directory_view)
 	fm_icon_view_update_smooth_graphics_mode (FM_ICON_VIEW (directory_view));
 }
 
-static void
-fm_icon_view_emblems_changed (FMDirectoryView *directory_view)
-{
-	g_assert (FM_IS_ICON_VIEW (directory_view));
-	
-	nautilus_icon_container_request_update_all (get_icon_container (FM_ICON_VIEW (directory_view)));	
-}
-
 static void
 default_sort_order_changed_callback (gpointer callback_data)
 {
@@ -2291,6 +2324,7 @@ icon_view_move_copy_items (NautilusIconContainer *container,
 static void
 fm_icon_view_update_icon_container_fonts (FMIconView *icon_view)
 {
+#if GNOME2_CONVERSION_COMPLETE
  	/* font size table - this isn't exactly proportional, but it looks better than computed */
 	static guint font_size_table[NAUTILUS_ZOOM_LEVEL_LARGEST + 1] = {
 		8, 8, 10, 12, 14, 18, 18 };
@@ -2309,6 +2343,7 @@ fm_icon_view_update_icon_container_fonts (FMIconView *icon_view)
 	}
 
 	nautilus_icon_container_request_update_all (icon_container);
+#endif
 }
 
 static int default_zoom_level_font_size = 12;
@@ -2382,25 +2417,6 @@ fm_icon_view_update_icon_container_font_size_table (FMIconView *icon_view)
 	nautilus_icon_container_request_update_all (icon_container);
 }
 
-static void
-fm_icon_view_update_icon_container_smooth_font (FMIconView *icon_view)
-{
-	NautilusIconContainer *icon_container;
-	EelScalableFont *scalable_font;
-
-	icon_container = get_icon_container (icon_view);
-	g_assert (icon_container != NULL);
-	
-	scalable_font = nautilus_global_preferences_get_icon_view_smooth_font ();
-	g_assert (EEL_IS_SCALABLE_FONT (scalable_font));
-
-	nautilus_icon_container_set_smooth_label_font (icon_container,
-						       scalable_font);
-	gtk_object_unref (GTK_OBJECT (scalable_font));
-	
-	nautilus_icon_container_request_update_all (icon_container);
-}
-
 static void
 fm_icon_view_update_click_mode (FMIconView *icon_view)
 {
@@ -2442,104 +2458,103 @@ create_icon_container (FMIconView *icon_view)
 
 	GTK_WIDGET_SET_FLAGS (icon_container, GTK_CAN_FOCUS);
 	
-	gtk_signal_connect (GTK_OBJECT (icon_container),
+	g_signal_connect (icon_container,
 			    "activate",
-			    GTK_SIGNAL_FUNC (icon_container_activate_callback),
+			    G_CALLBACK (icon_container_activate_callback),
 			    icon_view);
-	gtk_signal_connect (GTK_OBJECT (icon_container),
+	g_signal_connect (icon_container,
 			    "band_select_started",
-			    GTK_SIGNAL_FUNC (band_select_started_callback),
+			    G_CALLBACK (band_select_started_callback),
 			    icon_view);
-	gtk_signal_connect (GTK_OBJECT (icon_container),
+	g_signal_connect (icon_container,
 			    "band_select_ended",
-			    GTK_SIGNAL_FUNC (band_select_ended_callback),
+			    G_CALLBACK (band_select_ended_callback),
 			    icon_view);
-	gtk_signal_connect (GTK_OBJECT (icon_container),
+	g_signal_connect (icon_container,
 			    "compare_icons",
-			    GTK_SIGNAL_FUNC (icon_container_compare_icons_callback),
+			    G_CALLBACK (icon_container_compare_icons_callback),
 			    icon_view);
-	gtk_signal_connect (GTK_OBJECT (icon_container),
+	g_signal_connect (icon_container,
 			    "compare_icons_by_name",
-			    GTK_SIGNAL_FUNC (icon_container_compare_icons_by_name_callback),
+			    G_CALLBACK (icon_container_compare_icons_by_name_callback),
 			    icon_view);
-	gtk_signal_connect (GTK_OBJECT (icon_container),
+	g_signal_connect (icon_container,
 			    "context_click_selection",
-			    GTK_SIGNAL_FUNC (icon_container_context_click_selection_callback),
+			    G_CALLBACK (icon_container_context_click_selection_callback),
 			    icon_view);
-	gtk_signal_connect (GTK_OBJECT (icon_container),
+	g_signal_connect (icon_container,
 			    "context_click_background",
-			    GTK_SIGNAL_FUNC (icon_container_context_click_background_callback),
+			    G_CALLBACK (icon_container_context_click_background_callback),
 			    icon_view);
-	gtk_signal_connect (GTK_OBJECT (icon_container),
+	g_signal_connect (icon_container,
 			    "icon_position_changed",
-			    GTK_SIGNAL_FUNC (icon_position_changed_callback),
+			    G_CALLBACK (icon_position_changed_callback),
 			    icon_view);
-	gtk_signal_connect (GTK_OBJECT (icon_container),
+	g_signal_connect (icon_container,
 			    "icon_text_changed",
-			    GTK_SIGNAL_FUNC (fm_icon_view_icon_text_changed_callback),
+			    G_CALLBACK (fm_icon_view_icon_text_changed_callback),
 			    icon_view);
-	gtk_signal_connect (GTK_OBJECT (icon_container),
+	g_signal_connect (icon_container,
 			    "selection_changed",
-			    GTK_SIGNAL_FUNC (selection_changed_callback),
+			    G_CALLBACK (selection_changed_callback),
 			    icon_view);
-	gtk_signal_connect (GTK_OBJECT (icon_container),
+	g_signal_connect (icon_container,
 			    "get_icon_images",
-			    GTK_SIGNAL_FUNC (get_icon_images_callback),
+			    G_CALLBACK (get_icon_images_callback),
 			    icon_view);
-	gtk_signal_connect (GTK_OBJECT (icon_container),
+	g_signal_connect (icon_container,
 			    "get_icon_uri",
-			    GTK_SIGNAL_FUNC (get_icon_uri_callback),
+			    G_CALLBACK (get_icon_uri_callback),
 			    icon_view);
-	gtk_signal_connect (GTK_OBJECT (icon_container),
+	g_signal_connect (icon_container,
 			    "get_icon_drop_target_uri",
-			    GTK_SIGNAL_FUNC (get_icon_drop_target_uri_callback),
+			    G_CALLBACK (get_icon_drop_target_uri_callback),
 			    icon_view);
-	gtk_signal_connect (GTK_OBJECT (icon_container),
+	g_signal_connect (icon_container,
 			    "get_icon_text",
-			    GTK_SIGNAL_FUNC (get_icon_text_callback),
+			    G_CALLBACK (get_icon_text_callback),
 			    icon_view);
-	gtk_signal_connect (GTK_OBJECT (icon_container),
+	g_signal_connect (icon_container,
 			    "move_copy_items",
-			    GTK_SIGNAL_FUNC (icon_view_move_copy_items),
+			    G_CALLBACK (icon_view_move_copy_items),
 			    directory_view);
-	gtk_signal_connect (GTK_OBJECT (icon_container),
+	g_signal_connect (icon_container,
 			    "get_container_uri",
-			    GTK_SIGNAL_FUNC (icon_view_get_container_uri),
+			    G_CALLBACK (icon_view_get_container_uri),
 			    directory_view);
-	gtk_signal_connect (GTK_OBJECT (icon_container),
+	g_signal_connect (icon_container,
 			    "can_accept_item",
-			    GTK_SIGNAL_FUNC (icon_view_can_accept_item),
+			    G_CALLBACK (icon_view_can_accept_item),
 			    directory_view);
-	gtk_signal_connect (GTK_OBJECT (icon_container),
+	g_signal_connect (icon_container,
 			    "get_stored_icon_position",
-			    GTK_SIGNAL_FUNC (get_stored_icon_position_callback),
+			    G_CALLBACK (get_stored_icon_position_callback),
 			    directory_view);
-	gtk_signal_connect (GTK_OBJECT (icon_container),
+	g_signal_connect (icon_container,
 			    "layout_changed",
-			    GTK_SIGNAL_FUNC (layout_changed_callback),
+			    G_CALLBACK (layout_changed_callback),
 			    directory_view);
-	gtk_signal_connect (GTK_OBJECT (icon_container),
+	g_signal_connect (icon_container,
 			    "preview",
-			    GTK_SIGNAL_FUNC (icon_container_preview_callback),
+			    G_CALLBACK (icon_container_preview_callback),
 			    icon_view);
-	gtk_signal_connect (GTK_OBJECT (icon_container),
+	g_signal_connect (icon_container,
 			    "renaming_icon",
-			    renaming_icon_callback,
+			    G_CALLBACK (renaming_icon_callback),
 			    directory_view);
 	gtk_signal_connect_object (GTK_OBJECT (icon_container),
 			           "icon_stretch_started",
-			           fm_directory_view_update_menus,
+			           G_CALLBACK (fm_directory_view_update_menus),
 			           GTK_OBJECT (directory_view));
 	gtk_signal_connect_object (GTK_OBJECT (icon_container),
 			           "icon_stretch_ended",
-			           fm_directory_view_update_menus,
+			           G_CALLBACK (fm_directory_view_update_menus),
 			           GTK_OBJECT (directory_view));
 
 	gtk_container_add (GTK_CONTAINER (icon_view),
 			   GTK_WIDGET (icon_container));
 
 	fm_icon_view_update_icon_container_fonts (icon_view);
-	fm_icon_view_update_icon_container_smooth_font (icon_view);
 	fm_icon_view_update_icon_container_font_size_table (icon_view);
 	fm_icon_view_update_click_mode (icon_view);
 	fm_icon_view_update_smooth_graphics_mode (icon_view);
@@ -2552,10 +2567,16 @@ icon_view_handle_uri_list (NautilusIconContainer *container, const char *item_ur
 			   GdkDragAction action, int x, int y, FMIconView *view)
 {
 
-	GList *uri_list, *node, *real_uri_list = NULL;
+#if GNOME2_CONVERSION_COMPLETE
+	GList *uri_list;
+#endif
+	GList *node, *real_uri_list = NULL;
 	GnomeVFSURI *container_uri;
+#if GNOME2_CONVERSION_COMPLETE
 	GnomeDesktopEntry *entry;
+#endif
 	GdkPoint point;
+	char *uri;
 	char *local_path;
 	char *stripped_uri;
 	char *container_uri_string;
@@ -2611,7 +2632,8 @@ icon_view_handle_uri_list (NautilusIconContainer *container, const char *item_ur
 	 */
 	all_local = TRUE;
 	n_uris = 0;
-	uri_list = gnome_uri_list_extract_uris (item_uris);
+#if GNOME2_CONVERSION_COMPLETE
+	uri_list = gnome_vfs_uri_list_extract_uris (item_uris);
 	for (node = uri_list; node != NULL; node = node->next) {
 		gchar *sanitized_uri;
 
@@ -2623,7 +2645,8 @@ icon_view_handle_uri_list (NautilusIconContainer *container, const char *item_ur
 			all_local = FALSE;
 		n_uris++;
 	}
-	gnome_uri_list_free_strings (uri_list);
+	gnome_vfs_uri_list_free_strings (uri_list);
+#endif
 
 	if (all_local == TRUE &&
 	    (action == GDK_ACTION_COPY ||
@@ -2648,11 +2671,12 @@ icon_view_handle_uri_list (NautilusIconContainer *container, const char *item_ur
 		container_path = gnome_vfs_get_local_path_from_uri (container_uri_string);
 		for (node = real_uri_list; node != NULL; node = node->next) {
 			/* Make a link using the desktop file contents? */
-			local_path = gnome_vfs_get_local_path_from_uri (node->data);
+			uri = node->data;
+			local_path = gnome_vfs_get_local_path_from_uri (uri);
 			if (local_path != NULL) {
+#if GNOME2_CONVERSION_COMPLETE
 				entry = gnome_desktop_entry_load (local_path);
 				if (entry != NULL) {
-					
 					/* FIXME: Handle name conflicts? */
 					nautilus_link_local_create_from_gnome_entry (entry, container_path, &point);
 					gnome_desktop_entry_free (entry);
@@ -2661,6 +2685,7 @@ icon_view_handle_uri_list (NautilusIconContainer *container, const char *item_ur
 				if (entry != NULL) {
 					continue;
 				}
+#endif
 			}
 
 			/* Make a link from the URI alone. Generate the file
@@ -2669,41 +2694,40 @@ icon_view_handle_uri_list (NautilusIconContainer *container, const char *item_ur
 			/* FIXME: This should be using eel_uri_get_basename
 			 * instead of a "roll our own" solution.
 			 */
-			stripped_uri = eel_str_strip_trailing_chr ((char *)node->data, '/');
-			g_print ("local_path:%s\nstripped_uri:%s\n", (char *)node->data, stripped_uri);
+			stripped_uri = eel_str_strip_trailing_chr (uri, '/');
+			g_print ("local_path:%s\nstripped_uri:%s\n", local_path, stripped_uri);
 			last_slash = strrchr (stripped_uri, '/');
 			link_name = last_slash == NULL ? NULL : last_slash + 1;
 			
 			if (!eel_str_is_empty (link_name)) {
 				/* FIXME: Handle name conflicts? */
 				nautilus_link_local_create (container_path, link_name,
-							    "gnome-http-url", local_path,
+							    "gnome-http-url", uri,
 							    &point, NAUTILUS_LINK_GENERIC);
 			}
 			
 			g_free (stripped_uri);
-
 			break;
 		}
 		g_free (container_path);
 	}
 	
-	gnome_uri_list_free_strings (real_uri_list);
+#if GNOME2_CONVERSION_COMPLETE
+	gnome_vfs_uri_list_free_strings (real_uri_list);
+#endif
 	gnome_vfs_uri_unref (container_uri);
 	g_free (container_uri_string);
 	
 }
 
 static void
-fm_icon_view_initialize_class (FMIconViewClass *klass)
+fm_icon_view_class_init (FMIconViewClass *klass)
 {
-	GtkObjectClass *object_class;
 	FMDirectoryViewClass *fm_directory_view_class;
 
-	object_class = GTK_OBJECT_CLASS (klass);
 	fm_directory_view_class = FM_DIRECTORY_VIEW_CLASS (klass);
 
-	object_class->destroy = fm_icon_view_destroy;
+	G_OBJECT_CLASS (klass)->finalize = fm_icon_view_finalize;
 	
 	fm_directory_view_class->add_file = fm_icon_view_add_file;
 	fm_directory_view_class->begin_loading = fm_icon_view_begin_loading;
@@ -2730,7 +2754,6 @@ fm_icon_view_initialize_class (FMIconViewClass *klass)
         fm_directory_view_class->image_display_policy_changed = fm_icon_view_image_display_policy_changed;
         fm_directory_view_class->merge_menus = fm_icon_view_merge_menus;
         fm_directory_view_class->smooth_graphics_mode_changed = fm_icon_view_smooth_graphics_mode_changed;
-        fm_directory_view_class->emblems_changed = fm_icon_view_emblems_changed;
         fm_directory_view_class->sort_directories_first_changed = fm_icon_view_sort_directories_first_changed;
         fm_directory_view_class->start_renaming_item = fm_icon_view_start_renaming_item;
         fm_directory_view_class->text_attribute_names_changed = fm_icon_view_text_attribute_names_changed;
@@ -2752,7 +2775,7 @@ fm_icon_view_initialize_class (FMIconViewClass *klass)
 }
 
 static void
-fm_icon_view_initialize (FMIconView *icon_view)
+fm_icon_view_init (FMIconView *icon_view)
 {
 	NautilusIconContainer *icon_container;
 
@@ -2766,40 +2789,36 @@ fm_icon_view_initialize (FMIconView *icon_view)
 	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_ICON_VIEW_FONT,
 						  font_changed_callback, 
 						  icon_view,
-						  GTK_OBJECT (icon_view));
-	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_ICON_VIEW_SMOOTH_FONT,
-						  smooth_font_changed_callback, 
-						  icon_view,
-						  GTK_OBJECT (icon_view));
+						  G_OBJECT (icon_view));
 	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_ZOOM_LEVEL_FONT_SIZE,
 						  default_zoom_level_font_size_changed_callback,
 						  icon_view,
-						  GTK_OBJECT (icon_view));
+						  G_OBJECT (icon_view));
 	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_SORT_ORDER,
 						  default_sort_order_changed_callback,
 						  icon_view,
-						  GTK_OBJECT (icon_view));
+						  G_OBJECT (icon_view));
 	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_SORT_IN_REVERSE_ORDER,
 						  default_sort_in_reverse_order_changed_callback,
 						  icon_view,
-						  GTK_OBJECT (icon_view));
+						  G_OBJECT (icon_view));
 	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_USE_TIGHTER_LAYOUT,
 						  default_use_tighter_layout_changed_callback,
 						  icon_view,
-						  GTK_OBJECT (icon_view));
+						  G_OBJECT (icon_view));
 	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_USE_MANUAL_LAYOUT,
 						  default_use_manual_layout_changed_callback,
 						  icon_view,
-						  GTK_OBJECT (icon_view));
+						  G_OBJECT (icon_view));
 	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_ZOOM_LEVEL,
 						  default_zoom_level_changed_callback,
 						  icon_view,
-						  GTK_OBJECT (icon_view));
+						  G_OBJECT (icon_view));
 
 	icon_container = get_icon_container (icon_view);
-	gtk_signal_connect (GTK_OBJECT (icon_container),
+	g_signal_connect (icon_container,
 			    "handle_uri_list",
-			    GTK_SIGNAL_FUNC (icon_view_handle_uri_list),
+			    G_CALLBACK (icon_view_handle_uri_list),
 			    icon_view);
 
 }
