@@ -30,33 +30,28 @@
 #include "nautilus-sidebar.h"
 #include "nautilus-window.h"
 
-#include <ctype.h>
-#include <bonobo/bonobo-property-bag-client.h>
 #include <bonobo/bonobo-exception.h>
+#include <bonobo/bonobo-property-bag-client.h>
+#include <eel/eel-background.h>
+#include <eel/eel-gdk-extensions.h>
+#include <eel/eel-gdk-pixbuf-extensions.h>
+#include <eel/eel-glib-extensions.h>
+#include <eel/eel-gtk-extensions.h>
+#include <eel/eel-gtk-macros.h>
+#include <eel/eel-string.h>
 #include <gtk/gtkhbox.h>
+#include <gtk/gtkimage.h>
 #include <gtk/gtklabel.h>
-#include <gtk/gtkpixmap.h>
 #include <gtk/gtksignal.h>
 #include <gtk/gtkwidget.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnomevfs/gnome-vfs-types.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
-#include <eel/eel-background.h>
 #include <libnautilus-private/nautilus-file-attributes.h>
-#include <libnautilus-private/nautilus-font-factory.h>
-#include <eel/eel-gdk-extensions.h>
-#include <eel/eel-gdk-font-extensions.h>
-#include <eel/eel-gdk-pixbuf-extensions.h>
-#include <eel/eel-glib-extensions.h>
 #include <libnautilus-private/nautilus-global-preferences.h>
-#include <eel/eel-gtk-extensions.h>
-#include <eel/eel-gtk-macros.h>
 #include <libnautilus-private/nautilus-icon-factory.h>
-#include <eel/eel-image-with-background.h>
-#include <eel/eel-label-with-background.h>
 #include <libnautilus-private/nautilus-metadata.h>
 #include <libnautilus-private/nautilus-search-uri.h>
-#include <eel/eel-string.h>
 #include <libnautilus-private/nautilus-theme.h>
 #include <math.h>
 #include <string.h>
@@ -71,9 +66,9 @@
 #define MAX_TITLE_FONT_SIZE 	 20
 #define TITLE_PADDING		  4
 
-static void                nautilus_sidebar_title_initialize_class (NautilusSidebarTitleClass *klass);
+static void                nautilus_sidebar_title_class_init (NautilusSidebarTitleClass *klass);
 static void                nautilus_sidebar_title_destroy          (GtkObject                 *object);
-static void                nautilus_sidebar_title_initialize       (NautilusSidebarTitle      *pixmap);
+static void                nautilus_sidebar_title_init       (NautilusSidebarTitle      *pixmap);
 static void                nautilus_sidebar_title_size_allocate    (GtkWidget                 *widget,
 								    GtkAllocation             *allocation);
 static void                nautilus_sidebar_title_theme_changed    (gpointer                   user_data);
@@ -100,10 +95,10 @@ struct NautilusSidebarTitleDetails {
 	gboolean		determined_icon;
 };
 
-EEL_DEFINE_CLASS_BOILERPLATE (NautilusSidebarTitle, nautilus_sidebar_title, gtk_vbox_get_type ())
+EEL_CLASS_BOILERPLATE (NautilusSidebarTitle, nautilus_sidebar_title, gtk_vbox_get_type ())
 
 static void
-nautilus_sidebar_title_initialize_class (NautilusSidebarTitleClass *class)
+nautilus_sidebar_title_class_init (NautilusSidebarTitleClass *class)
 {
 	GtkObjectClass *object_class;
 	GtkWidgetClass *widget_class;
@@ -135,31 +130,12 @@ realize_callback (NautilusSidebarTitle *sidebar_title)
 
 	gtk_signal_connect_while_alive (GTK_OBJECT (background),
 					"appearance_changed",
-					appearance_changed_callback,
+					G_CALLBACK (appearance_changed_callback),
 					sidebar_title,
 					GTK_OBJECT (sidebar_title));
 }
 
-static void
-smooth_font_changed_callback (gpointer callback_data)
-{
-	EelScalableFont *new_font;
-	EelScalableFont *new_bold_font;
-	NautilusSidebarTitle *sidebar_title;
-
-	g_return_if_fail (NAUTILUS_IS_SIDEBAR_TITLE (callback_data));
-
-	sidebar_title = NAUTILUS_SIDEBAR_TITLE (callback_data);
-	
-	new_font = nautilus_global_preferences_get_default_smooth_font ();
-	new_bold_font = nautilus_global_preferences_get_default_smooth_bold_font ();
-
-	eel_label_set_smooth_font (EEL_LABEL (sidebar_title->details->title_label), new_bold_font);
-	eel_label_set_smooth_font (EEL_LABEL (sidebar_title->details->more_info_label), new_font);
-
-	gtk_object_unref (GTK_OBJECT (new_font));
-	gtk_object_unref (GTK_OBJECT (new_bold_font));
-}
+#if GNOME2_CONVERSION_COMPLETE
 
 static GdkFont *
 get_non_smooth_font (int font_size)
@@ -192,20 +168,23 @@ non_smooth_font_changed_callback (gpointer callback_data)
 	gdk_font_unref (new_font);
 }
 
+#endif
+
 static void
-nautilus_sidebar_title_initialize (NautilusSidebarTitle *sidebar_title)
+nautilus_sidebar_title_init (NautilusSidebarTitle *sidebar_title)
 { 
 	sidebar_title->details = g_new0 (NautilusSidebarTitleDetails, 1);
 
 	/* Register to find out about icon theme changes */
 	gtk_signal_connect_object_while_alive (nautilus_icon_factory_get (),
 					       "icons_changed",
-					       update_icon,
+					       G_CALLBACK (update_icon),
 					       GTK_OBJECT (sidebar_title));
-	gtk_signal_connect (GTK_OBJECT (sidebar_title), "realize", realize_callback, NULL);
+	g_signal_connect (sidebar_title, "realize",
+			    G_CALLBACK (realize_callback), NULL);
 
 	/* Create the icon */
-	sidebar_title->details->icon = eel_image_new_with_background (NULL);
+	sidebar_title->details->icon = gtk_image_new ();
 	gtk_box_pack_start (GTK_BOX (sidebar_title), sidebar_title->details->icon, 0, 0, 0);
 	gtk_widget_show (sidebar_title->details->icon);
 
@@ -223,11 +202,6 @@ nautilus_sidebar_title_initialize (NautilusSidebarTitle *sidebar_title)
 	gtk_widget_show (sidebar_title->details->emblem_box);
 	gtk_box_pack_start (GTK_BOX (sidebar_title), sidebar_title->details->emblem_box, 0, 0, 0);
 
-	/* FIXME: This should use EelLabel like the other displayed text.
-	 * But I don't think this feature is ever used? Someone should consult
-	 * with Andy about this. (This is not the same as the info in the Notes
-	 * sidebar panel.)
-	 */
 	sidebar_title->details->notes = GTK_WIDGET (gtk_label_new (NULL));
 	gtk_label_set_line_wrap (GTK_LABEL (sidebar_title->details->notes), TRUE);
 	gtk_widget_show (sidebar_title->details->notes);
@@ -235,23 +209,22 @@ nautilus_sidebar_title_initialize (NautilusSidebarTitle *sidebar_title)
 
 	/* Keep track of changes in graphics trade offs */
 	update_all (sidebar_title);
+#if GNOME2_CONVERSION_COMPLETE
 	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_DEFAULT_FONT,
-						       non_smooth_font_changed_callback,
-						       sidebar_title,
-						       GTK_OBJECT (sidebar_title));
-	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_DEFAULT_SMOOTH_FONT,
-						       smooth_font_changed_callback,
-						       sidebar_title,
-						       GTK_OBJECT (sidebar_title));
+						  non_smooth_font_changed_callback,
+						  sidebar_title,
+						  G_OBJECT (sidebar_title));
+#endif
 	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_THEME,
-						       nautilus_sidebar_title_theme_changed,
-						       sidebar_title,
-						       GTK_OBJECT (sidebar_title));
+						  nautilus_sidebar_title_theme_changed,
+						  sidebar_title,
+						  G_OBJECT (sidebar_title));
 
 	/* initialize the label colors & fonts */
 	nautilus_sidebar_title_theme_changed (sidebar_title);
-	smooth_font_changed_callback (sidebar_title);
+#if GNOME2_CONVERSION_COMPLETE
 	non_smooth_font_changed_callback (sidebar_title);
+#endif
 }
 
 /* destroy by throwing away private storage */
@@ -278,10 +251,13 @@ nautilus_sidebar_title_destroy (GtkObject *object)
 
 	sidebar_title = NAUTILUS_SIDEBAR_TITLE (object);
 
-	release_file (sidebar_title);
+	if (sidebar_title->details) {
+		release_file (sidebar_title);
 	
-	g_free (sidebar_title->details->title_text);
-	g_free (sidebar_title->details);
+		g_free (sidebar_title->details->title_text);
+		g_free (sidebar_title->details);
+		sidebar_title->details = NULL;
+	}
 
 	EEL_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
 }
@@ -372,41 +348,39 @@ nautilus_sidebar_title_select_text_color (NautilusSidebarTitle *sidebar_title)
 		if (sidebar_title_color == NULL) {
 			/* FIXME bugzilla.gnome.org 42496: for now, both the title and info colors are the same */
 			if (eel_background_is_dark (background)) {
-				sidebar_title_color = g_strdup("rgb:FFFF/FFFF/FFFF");
-				sidebar_info_title_color = g_strdup("rgb:FFFF/FFFF/FFFF");
-				sidebar_title_shadow_color = g_strdup("rgb:0000/0000/0000");
+				sidebar_title_color = g_strdup ("#FFFFFF");
+				sidebar_info_title_color = g_strdup ("#FFFFFF");
+				sidebar_title_shadow_color = g_strdup ("#000000");
 				
 			} else {
-				sidebar_title_color = g_strdup("rgb:0000/0000/0000");
-				sidebar_info_title_color = g_strdup("rgb:0000/0000/0000");
-				sidebar_title_shadow_color = g_strdup("rgb:FFFF/FFFF/FFFF");
+				sidebar_title_color = g_strdup ("#000000");
+				sidebar_info_title_color = g_strdup ("#000000");
+				sidebar_title_shadow_color = g_strdup ("#FFFFFF");
 			}
 		} else {
 			if (sidebar_info_title_color == NULL) {
 				sidebar_info_title_color = g_strdup (sidebar_title_color);
 			}
 			if (sidebar_title_shadow_color == NULL) {
-				sidebar_title_shadow_color = g_strdup("rgb:FFFF/FFFF/FFFF");
+				sidebar_title_shadow_color = g_strdup ("#FFFFFF");
 			}
 		}
 
-		eel_label_set_text_color (EEL_LABEL (sidebar_title->details->title_label),
-						       eel_parse_rgb_with_white_default (sidebar_title_color));
-		
+		eel_gtk_widget_set_foreground_color (sidebar_title->details->title_label,
+						     sidebar_title_color);
+		eel_gtk_widget_set_foreground_color (sidebar_title->details->more_info_label,
+						     sidebar_info_title_color);
+
+#if GNOME2_CONVERSION_COMPLETE
 		eel_label_set_smooth_drop_shadow_color (EEL_LABEL (sidebar_title->details->title_label),
-							      eel_parse_rgb_with_white_default (sidebar_title_shadow_color));
-		
-		eel_label_set_smooth_drop_shadow_offset (EEL_LABEL (sidebar_title->details->title_label),
-							       sidebar_title->details->shadow_offset);
-		
-		eel_label_set_text_color (EEL_LABEL (sidebar_title->details->more_info_label),
-						       eel_parse_rgb_with_white_default (sidebar_info_title_color));
-		
+							eel_parse_rgb_with_white_default (sidebar_title_shadow_color));
 		eel_label_set_smooth_drop_shadow_color (EEL_LABEL (sidebar_title->details->more_info_label),
-							      eel_parse_rgb_with_white_default (sidebar_title_shadow_color));
-		
+							eel_parse_rgb_with_white_default (sidebar_title_shadow_color));
+		eel_label_set_smooth_drop_shadow_offset (EEL_LABEL (sidebar_title->details->title_label),
+							 sidebar_title->details->shadow_offset);
 		eel_label_set_smooth_drop_shadow_offset (EEL_LABEL (sidebar_title->details->more_info_label),
-							       sidebar_title->details->shadow_offset);
+							 sidebar_title->details->shadow_offset);
+#endif
 		
 		g_free (sidebar_title_color);	
 		g_free (sidebar_info_title_color);	
@@ -495,12 +469,11 @@ update_icon (NautilusSidebarTitle *sidebar_title)
 
 	pixbuf = NULL;
 	if (icon_name != NULL && strlen (icon_name) > 0) {
-		pixbuf = nautilus_icon_factory_get_pixbuf_from_name (icon_name, NULL, NAUTILUS_ICON_SIZE_LARGE, TRUE);
+		pixbuf = nautilus_icon_factory_get_pixbuf_from_name (icon_name, NULL, NAUTILUS_ICON_SIZE_LARGE);
 	} else if (nautilus_icon_factory_is_icon_ready_for_file (sidebar_title->details->file)) {
 		pixbuf = nautilus_icon_factory_get_pixbuf_for_file (sidebar_title->details->file,
 								    "accept",
-								    NAUTILUS_ICON_SIZE_LARGE,
-								    TRUE);
+								    NAUTILUS_ICON_SIZE_LARGE);
 	} else if (sidebar_title->details->determined_icon) {
 		/* We used to know the icon for this file, but now the file says it isn't
 		 * ready. This means that some file info has been invalidated, which
@@ -519,7 +492,7 @@ update_icon (NautilusSidebarTitle *sidebar_title)
 	}
 
 	if (!leave_pixbuf_unchanged) {
-		eel_image_set_pixbuf (EEL_IMAGE (sidebar_title->details->icon), pixbuf);
+		gtk_image_set_from_pixbuf (GTK_IMAGE (sidebar_title->details->icon), pixbuf);
 	}
 }
 
@@ -527,11 +500,13 @@ static void
 update_title_font (NautilusSidebarTitle *sidebar_title)
 {
 	int available_width;
+#if GNOME2_CONVERSION_COMPLETE
 	GdkFont *template_font;
 	GdkFont *bold_template_font;
 	GdkFont *largest_fitting_font;
 	int largest_fitting_smooth_font_size;
 	EelScalableFont *smooth_font;
+#endif
 
 	/* Make sure theres work to do */
 	if (eel_strlen (sidebar_title->details->title_text) < 1) {
@@ -545,6 +520,7 @@ update_title_font (NautilusSidebarTitle *sidebar_title)
 		return;
 	}
 	
+#if GNOME2_CONVERSION_COMPLETE
 	/* Update the smooth font */
 	smooth_font = eel_label_get_smooth_font (EEL_LABEL (sidebar_title->details->title_label));
 	largest_fitting_smooth_font_size = eel_scalable_font_largest_fitting_font_size
@@ -557,7 +533,7 @@ update_title_font (NautilusSidebarTitle *sidebar_title)
 	eel_label_set_smooth_font_size (EEL_LABEL (sidebar_title->details->title_label), 
 					largest_fitting_smooth_font_size);
 	
-	gtk_object_unref (GTK_OBJECT (smooth_font));
+	g_object_unref (smooth_font);
 
 	/* Update the regular font */
 	template_font = get_non_smooth_font (MAX_TITLE_FONT_SIZE);
@@ -575,21 +551,32 @@ update_title_font (NautilusSidebarTitle *sidebar_title)
 	}
 	
 	eel_gtk_widget_set_font (sidebar_title->details->title_label,
-				      largest_fitting_font);
+				 largest_fitting_font);
 	
 	gdk_font_unref (largest_fitting_font);
 	gdk_font_unref (bold_template_font);
 	gdk_font_unref (template_font);
+#else
+	eel_gtk_label_make_bold (GTK_LABEL (sidebar_title->details->title_label));
+	eel_gtk_label_set_scale (GTK_LABEL (sidebar_title->details->title_label), PANGO_SCALE_LARGE);
+#endif
 }
 
 static void
 update_title (NautilusSidebarTitle *sidebar_title)
 {
+	GtkLabel *label;
+	const char *text;
+
+	label = GTK_LABEL (sidebar_title->details->title_label);
+	text = sidebar_title->details->title_text;
+
 	/* FIXME bugzilla.gnome.org 42500: We could defer showing the title until the icon is ready. */
-	if (eel_label_set_text (EEL_LABEL (sidebar_title->details->title_label),
-				     sidebar_title->details->title_text)) {
-		update_title_font (sidebar_title);
+	if (eel_strcmp (text, gtk_label_get_text (label)) == 0) {
+		return;
 	}
+	gtk_label_set_text (label, text);
+	update_title_font (sidebar_title);
 }
 
 static void
@@ -619,16 +606,15 @@ file_is_search_location (NautilusFile *file)
 }
 
 static int
-measure_width_callback (const char *string, void *context)
+measure_width_callback (const char *string, gpointer callback_data)
 {
-	EelLabel *label;
-	EelScalableFont *smooth_font;
-	int smooth_font_size;
-	
-	label = (EelLabel*) context;
-	smooth_font = eel_label_get_smooth_font (label); 
-	smooth_font_size = eel_label_get_smooth_font_size (label);
-	return eel_scalable_font_text_width (smooth_font, smooth_font_size, string, strlen (string));
+	PangoLayout *layout;
+	int width;
+
+	layout = PANGO_LAYOUT (callback_data);
+	pango_layout_set_text (layout, string, -1);
+	pango_layout_get_pixel_size (layout, &width, NULL);
+	return width;
 }
 
 static void
@@ -640,6 +626,7 @@ update_more_info (NautilusSidebarTitle *sidebar_title)
 	char *search_string, *search_uri;
 	char *date_modified_str;
 	int sidebar_width;
+	PangoLayout *layout;
 	
 	file = sidebar_title->details->file;
 
@@ -674,15 +661,16 @@ update_more_info (NautilusSidebarTitle *sidebar_title)
 			
 			sidebar_width = GTK_WIDGET (sidebar_title)->allocation.width - 2 * SIDEBAR_INFO_MARGIN;
 			if (sidebar_width > MINIMUM_INFO_WIDTH) {
-				date_modified_str = nautilus_file_fit_modified_date_as_string (file, sidebar_width,
-											measure_width_callback, NULL,
-											sidebar_title->details->more_info_label);
+				layout = pango_layout_copy (gtk_label_get_layout (GTK_LABEL (sidebar_title->details->more_info_label)));
+				pango_layout_set_width (layout, -1);
+				date_modified_str = nautilus_file_fit_modified_date_as_string
+					(file, sidebar_width, measure_width_callback, NULL, layout);
+				g_object_unref (layout);
 				append_and_eat (info_string, "\n", date_modified_str);
-				g_string_append_c (info_string, '\0');
 			}
 		}
 	}
-	eel_label_set_text (EEL_LABEL (sidebar_title->details->more_info_label),
+	gtk_label_set_text (GTK_LABEL (sidebar_title->details->more_info_label),
 			    info_string->str);
 
 	g_string_free (info_string, TRUE);
@@ -694,8 +682,7 @@ add_emblem (NautilusSidebarTitle *sidebar_title, GdkPixbuf *pixbuf)
 {
 	GtkWidget *image_widget;
 
-	image_widget = eel_image_new_with_background (NULL);
-	eel_image_set_pixbuf (EEL_IMAGE (image_widget), pixbuf);	
+	image_widget = gtk_image_new_from_pixbuf (pixbuf);
   	gtk_widget_show (image_widget);
 	gtk_container_add (GTK_CONTAINER (sidebar_title->details->emblem_box), image_widget);	
 }
@@ -726,10 +713,10 @@ update_emblems (NautilusSidebarTitle *sidebar_title)
 			(p->data,
 			 NAUTILUS_ICON_SIZE_STANDARD, NAUTILUS_ICON_SIZE_STANDARD,
 			 NAUTILUS_ICON_SIZE_STANDARD, NAUTILUS_ICON_SIZE_STANDARD,
-			 FALSE, NULL, FALSE);
+			 NULL, FALSE);
 		if (pixbuf != NULL) {
 			add_emblem (sidebar_title, pixbuf);
-			gdk_pixbuf_unref (pixbuf);
+			g_object_unref (pixbuf);
 		}
 	}
 	
@@ -805,7 +792,7 @@ nautilus_sidebar_title_set_file (NautilusSidebarTitle *sidebar_title,
 			sidebar_title->details->file_changed_connection =
 				gtk_signal_connect_object (GTK_OBJECT (sidebar_title->details->file),
 							   "changed",
-							   update_all,
+							   G_CALLBACK (update_all),
 							   GTK_OBJECT (sidebar_title));
 			
 			/* Monitor the things needed to get the right
@@ -860,10 +847,10 @@ sidebar_title_create_title_label (void)
 { 
 	GtkWidget *title_label;
 
-	title_label = eel_label_new_with_background ("");
-	eel_label_make_bold (EEL_LABEL (title_label));
-	eel_label_set_wrap (EEL_LABEL (title_label), TRUE);
-	eel_label_set_justify (EEL_LABEL (title_label), GTK_JUSTIFY_CENTER);
+	title_label = gtk_label_new ("");
+	eel_gtk_label_make_bold (GTK_LABEL (title_label));
+	gtk_label_set_line_wrap (GTK_LABEL (title_label), TRUE);
+	gtk_label_set_justify (GTK_LABEL (title_label), GTK_JUSTIFY_CENTER);
 
 	return title_label;
 }
@@ -873,10 +860,9 @@ sidebar_title_create_more_info_label (void)
 {
 	GtkWidget *more_info_label;
 
-	more_info_label = eel_label_new_with_background ("");
-	eel_label_make_smaller (EEL_LABEL (more_info_label), 2);
-	eel_label_set_justify (EEL_LABEL (more_info_label), GTK_JUSTIFY_CENTER);
-	eel_label_set_justify (EEL_LABEL (more_info_label), GTK_JUSTIFY_CENTER);
+	more_info_label = gtk_label_new ("");
+	eel_gtk_label_set_scale (GTK_LABEL (more_info_label), PANGO_SCALE_SMALL);
+	gtk_label_set_justify (GTK_LABEL (more_info_label), GTK_JUSTIFY_CENTER);
 	
 	return more_info_label;
 }
